##############################################################################
#    Copyright (C) 2002  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import os, zLOG, re, sys, traceback
from Globals import InitializeClass, package_home
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions
from Products.PageTemplates import PageTemplateFile
from DateTime import DateTime
from utils import getUniqueId

_zptdir = os.path.join(package_home(globals()), 'www')

class FormatObject(SimpleItem):
    """
    An object to define the formatting of the content of the boxes.
    """
    id = None
    title = None
    format_list = []
    sort_order = 0
    NEWLINE = "&lt;new line&gt;"
    HORIZONTALLINE = "&lt;horizontal line&gt;"
    TEXTLINE = "&lt;text line&gt;"

    manage_options =  ( { 'label' : 'Content', 'action' : 'manage_formatObject' }
                     ,
                     ) + SimpleItem.manage_options

    security = ClassSecurityInfo()
    type = 'Format Object'
    meta_type = 'Format Object'

    #
    #   ZMI methods
    #
    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_formatObject' )
    manage_formatObject = PageTemplateFile.PageTemplateFile( 'formatObjectLines', _zptdir)

    def __init__(self, id='', title=''):
        """
        Initiate a new format object
        """
        if id:
            rgx = re.compile(r'(^_|[^a-zA-Z0-9-_~\,\.])')
            id = re.sub(rgx, "",id)

        self.id = id or getUniqueId()
        self.title = title

    def __cmp__(self, other):
        """ Compare function - used when sorting elements """
        if other is not None:
            if hasattr(other, 'sort_order'):
                return cmp(self.sort_order, other.sort_order)
            else:
                return 1
        else:
            return 1

    security.declarePublic('listFormatList')
    def listElements(self):
        """
        Returns the formatting
        """
        return self.format_list

    security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_editElements' )
    def manage_editElements( self, properties=None, REQUEST=None ):

        """ Update our list of elements.
        """
        if properties is None:
            properties = REQUEST

        elements = []

        for index in range( len( self.format_list ) ):
            elements.append( self._extractFormat( properties, index ) )

        self.format_list = tuple( elements )

        if REQUEST is not None:
            return self.manage_formatObject(
                self, self.REQUEST, manage_tabs_message='Saved.')

    security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_edit')
    def manage_edit(self, title, REQUEST=None):
        """
        Edit the title of the formatting object
        """
        zLOG.LOG(self.meta_type, zLOG.INFO, 'manage_edit: title %s' % title)
        self.title = title

        #catalog = getToolByName(self, 'portal_catalog')
        #catalog.reindexObject(self)

        if REQUEST is not None:
            return self.manage_formatObject(
                self, self.REQUEST, manage_tabs_message='Saved.')

    security.declarePrivate( '_extractFormat' )
    def _extractFormat( self, properties, index ):

        """
        Extract a format element from the funky form properties.
        Got this from ActionProviderBase.
        """
        link = properties.get('link_%d' % index, None)
        style = str(properties.get('style_%d' % index, ''))
        type = str(properties.get('type_%d' % index, ''))

        if link == 'None':
            link = None
            
        element = {}
            
        if type == self.TEXTLINE:
            text = str(properties.get('txt_%d' % index, ''))
            element['text'] = text
        element['type'] = type
        element['style'] = style
        element['link'] = link

        return element

    security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_addElement')
    def manage_addElement(self, REQUEST):
        """
        Add a new format dictionary.
        """
        req = REQUEST.form
        ftype = req.get('type', None)
        if not ftype:
            raise 'Attribute Error', 'Type not selected'
        flink = req.get('link', None)
        if flink == 'None':
            flink = None
        dict = {'type': ftype
               ,'link': flink
               ,'style':''}
        formatlist = list(self.format_list)
        formatlist.append(dict)
        self.format_list = tuple(formatlist)
        self._p_changed = 1

        if not REQUEST.has_key('noredirect'):
            return self.manage_formatObject(
                self, self.REQUEST, manage_tabs_message='%s added.' % ftype)

    security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_addNewline')
    def manage_addNewline(self, REQUEST):
        """
        Add a new line.
        """
        req = REQUEST.form
        ftype = self.NEWLINE
        dict = {'type': ftype}
        formatlist = list(self.format_list)
        formatlist.append(dict)
        self.format_list = tuple(formatlist)
        self._p_changed = 1
        if not REQUEST.has_key('noredirect'):
            return self.manage_formatObject(
                self, self.REQUEST, manage_tabs_message='Newline added.')

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_addHorizontalLine')
    def manage_addHorizontalLine(self, REQUEST):
        """
        Add a new horizontal line.
        """
        req = REQUEST.form
        ftype = self.HORIZONTALLINE
        dict = {'type': ftype}
        formatlist = list(self.format_list)
        formatlist.append(dict)
        self.format_list = tuple(formatlist)
        self._p_changed = 1
        if not REQUEST.has_key('noredirect'):
            return self.manage_formatObject(self, self.REQUEST, manage_tabs_message='Horizontal line added.')
            
    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_addTextLine')
    def manage_addTextLine(self, REQUEST):
        """
        Add a new text line
        """
        req = REQUEST.form
        ftype = self.TEXTLINE
        ftext = req.get('line_text', '')
        dict = {'type': ftype
               ,'link': None
               ,'style':''
               ,'text': ftext}
        formatlist = list(self.format_list)
        formatlist.append(dict)
        self.format_list = tuple(formatlist)
        self._p_changed = 1
        if not REQUEST.has_key('noredirect'):
            return self.manage_formatObject(self, self.REQUEST, manage_tabs_message='Text line added.')

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_deleteElement')
    def manage_deleteElement(self, ids=(), REQUEST=None):
        """
        Delete the requested formatting
        """
        ids = list( map( int, ids ) )  # Convert to a list of integers.
        zLOG.LOG(self.meta_type, zLOG.INFO, 'deleteElement: ids %s' % str(ids))
        if ids:
            new_formats = []
            formatlist = list(self.format_list)
            for idx in range(len(formatlist)):
                if idx not in ids:
                    new_formats.append(formatlist[idx])
            self.format_list = tuple(new_formats)
            self._p_changed = 1
        if REQUEST:
            return self.manage_formatObject(
                self, self.REQUEST, manage_tabs_message='Element(s) deleted.')

    security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_moveUpElements' )
    def manage_moveUpElements( self, ids=(), REQUEST=None ):

        """ Move the specified elements up one slot in our list.
        """
        elements = list( map( int, ids ) )  # Convert to a list of integers.
        elements.sort()

        formatlist = list(self.format_list)

        for idx in elements:
            idx2 = idx - 1
            if idx2 < 0:
                # Wrap to the bottom.
                idx2 = len(formatlist) - 1
            # Swap.
            a = formatlist[idx2]
            formatlist[idx2] = formatlist[idx]
            formatlist[idx] = a

        self.format_list = tuple( formatlist )

        if REQUEST is not None:
            return self.manage_formatObject(
                self, self.REQUEST, manage_tabs_message='Moved.')

    security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_moveDownElements' )
    def manage_moveDownElements( self, ids=(), REQUEST=None ):

        """ Move the specified actions down one slot in our list.
        """
        elements = list( map( int, ids ) )  # Convert to a list of integers.
        elements.sort()
        elements.reverse()

        formatlist = list(self.format_list)

        for idx in elements:
            idx2 = idx + 1
            if idx2 >= len(formatlist):
                # Wrap to the top.
                idx2 = 0
            # Swap.
            a = formatlist[idx2]
            formatlist[idx2] = formatlist[idx]
            formatlist[idx] = a

        self.format_list = tuple( formatlist )

        if REQUEST:
            return self.manage_formatObject(
                self, self.REQUEST, manage_tabs_message='Moved.')


    def manage_afterAdd(self, item, container):
        """ set the sort order """
        zLOG.LOG('FormatObject', zLOG.INFO, 'manage_afterAdd()')
        sort_order = 0
        for e in container.objectValues():
            #zLOG.LOG('AbstractElement', zLOG.INFO, 'manage_afterAdd: elem %s' % (e.getId()))
            try:
                if e.getId() != item.getId() and e.sort_order >= sort_order:
                    sort_order = (e.sort_order + 1)
            except:
                zLOG.LOG(self.meta_type, zLOG.INFO, 'manage_afterAdd: exception %s - %s\n%s' % (sys.exc_type, sys.exc_value, traceback.format_list(traceback.extract_tb(sys.exc_traceback))))

        self.sort_order = sort_order

