##############################################################################
#    Copyright (C) 2002  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import os, zLOG, sys, traceback
from Globals import InitializeClass, package_home
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base, aq_parent, aq_inner, aq_chain
from OFS.SimpleItem import SimpleItem
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions
from LayoutPermissions import AddLayouts, ChangeLayouts
from DateTime import DateTime
from utils import getUniqueId
from LayoutTool import registerLayoutType

_zptdir = os.path.join(package_home(globals()), 'www')

class LayoutBase(SimpleItem):

    security = ClassSecurityInfo()
    sort_order = 0
    id = None
    title = None
    _render_title = 1
    _render_header = 0
    _language = None
    _showfor_type = 'both'
    _islocked = 0
    _valid_object_types = [{'id':'object', 'title': 'Object'}
                          ,{'id':'folder', 'title': 'Folder'}
                          ,{'id':'both', 'title':'Both folder and object'}
                          ]
    _renderInMain = 1

    def __call__(self):
        '''
        Invokes the default view.
        '''
        viewname = self.getViewMethod()
        view = self.restrictedTraverse(viewname)
        if getattr(aq_base(view), 'isDocTemp', 0):
            return apply(view, (self, self.REQUEST))
        else:
            return view()

    view = __call__

    index_html = None  # This special value informs ZPublisher to use __call__

    def renderAlone(self):
        """
        Invokes the standalone view (if any) or normal view
        """
        sa_name = self.getStandaloneMethod()
        if not sa_name:
            sa_name = self.getViewMethod()
        sa = self.restrictedTraverse(sa_name)
        if getattr(aq_base(sa), 'isDocTemp', 0):
            return apply(sa, (self, self.REQUEST))
        else:
            return sa()

    security.declarePublic('getId')
    def getId(self):
        """
        Return the id of the tab object.
        """
        return self.id

    def __cmp__(self, other):
        """ Compare function - used when sorting elements """
        if other is not None:
            return cmp(self.sort_order, other.sort_order)
        else:
            return 1

    security.declarePublic('getSlotName')
    def getSlotName(self):
        """
        Returns the name of the current slot.
        """
        return self.aq_parent.getId()[3:]

    security.declarePublic('getParent')
    def getParent(self):
        """
        Returns the parent container; not the layout container.
        """
        item = self
        _notfound = []
        layout_folder_id = self.aq_parent.id
        while 1:
            layouts = getattr(aq_base(item), layout_folder_id, _notfound)
            if layouts is not _notfound:
                break
            item = getattr(item, 'aq_parent', _notfound)
            if item is _notfound:
                break

        return item #self.REQUEST['PARENTS'][1]

    security.declarePublic('getEditMethod')
    def getEditMethod(self):
        """
        Returns the name of the edit form to use for this menu type.
        """
        return self._edit_method

    security.declarePublic('getViewMethod')
    def getViewMethod(self):
        """
        Returns the name of the edit form to use for this menu type.
        """
        return self._view_method

    security.declarePublic('getStandaloneMethod')
    def getStandaloneMethod(self):
        """
        Returns the name of the standalone form to use
        """
        return getattr(self, '_standalone_method', None)

    security.declarePublic('getMetaType')
    def getMetaType(self):
        """
        Returns the meta type of the tab object.
        """
        return self.meta_type

    security.declarePublic('Title')
    def Title(self):
        """
        Return the title of the tab object.
        """
        return self.title

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setTitle')
    def setTitle(self, title=None):
        """
        Update the title of the tab object.
        """
        if title is not None:
            self.title = title

    security.declarePrivate('setRenderTitle')
    def setRenderTitle(self, render=None):
        """
        Set if the title should be rendered in the menu
        """
        if render is not None:
            self._render_title = 1
        else:
            self._render_title = 0

    security.declarePublic('renderTitle')
    def renderTitle(self):
        """
        Return a boolean indicating whether the title should be rendered in the menu or not
        """
        return int(self._render_title)
        
    security.declarePrivate('setRenderHeader')
    def setRenderHeader(self, render=None):
        """
        Set if the header should be rendered in the menu
        """
        if render is not None:
            self._render_header = 1
        else:
            self._render_header = 0

    security.declarePublic('renderHeader')
    def renderHeader(self):
        """
        Return a boolean indicating whether the header should be rendered in the menu or not
        """
        return int(self._render_header)
        
    security.declareProtected(CMFCorePermissions.ManageProperties, 'setRenderInMain')
    def setRenderInMain(self, render=None):
        """
        Set if the layout should be rendered in main
        (on 'index_html') or not
        """
        if render is not None:
            self._renderInMain = 1
        else:
            self._renderInMain = 0
            
    security.declarePublic('renderInMain')
    def renderInMain(self):
        """
        Return a boolean indication whether the layout should be rendered in main slot
        (on 'index_html') or not
        """
        return int(self._renderInMain)
        
    security.declareProtected(CMFCorePermissions.ManageProperties, 'setLock')
    def setLock(self, lock=None):
        """
        Set a 'lock' on the layout item in portal top-level, in order to prevent
        the layout item to be overridden at lower levels of the portal
        """
        if lock is not None:
            self._islocked = 1
        else:
            self._islocked = 0

    security.declarePublic('isLocked')
    def isLocked(self):
        """
        Return a boolean indication whether the layout item is locked or it can
        be overriden at lower levels at the portal than in the portal root.
        """
        return int(self._islocked)

    security.declarePublic('getWidth')
    def getWidth(self):
        """
        Return the width of the tab object.
        """
        if hasattr(self, 'width'):
            return self.width
        else:
            return None

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setWidth')
    def setWidth(self, width=None):
        """
        Update the width of the tab object.
        """
        if width is not None:
            self.width = width

    security.declarePublic('getSortOrder')
    def getSortOrder(self):
        """
        Returns the sort order for the results
        """
        if hasattr(self, 'sort_order'):
            return self.sort_order
        else:
            return None

    security.declareProtected(ChangeLayouts, 'setSortOrder')
    def setSortOrder(self, sort_order=None):
        """
        Set the sort order for the results
        """
        self.sort_order = sort_order
        
    security.declareProtected(CMFCorePermissions.ManageProperties, 'setLanguage')
    def setLanguage(self, language):
        """
        Set the language that the item should turn up for
        """
        if language:
            if language == 'None':
                language = None
            self._language = language
            
    security.declarePublic('getLanguage')
    def getLanguage(self):
        """
        Return the language that the item should appear for
        """
        return self._language

    security.declarePublic('getMessageCatalog')
    def getMessageCatalog(self):
        """
        Return the message catalog
        """
        try:
            layout = getToolByName(self, 'portal_layout')
            mc = self.restrictedTraverse(layout.getMessageCatalog())
        except:
            zLOG.LOG('Calendar Box', zLOG.ERROR, 'Msg catalog error is %s - %s\n%s' % \
                            (sys.exc_info()[0]
                            , sys.exc_info()[1]
                            , traceback.format_list(traceback.extract_tb(sys.exc_traceback))
                            ))
            mc = None
        return mc

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setShowupFor')
    def setShowupFor(self, object_type=None):
        """
        Set what object type the box will appear for:
        Object, Folder, Both
        """
        if object_type is not None:
            self._showfor_type = object_type

    security.declarePublic('getShowupFor')
    def getShowupFor(self):
        """
        Return what object type the box shall appear for
        """
        return self._showfor_type

    security.declarePublic('validShowForTypes')
    def validShowForTypes(self):
        """
        Return a list containing dictionaries for valid object types
        """
        return self._valid_object_types

    security.declarePublic('showForThis')
    def showForThis(self):
        """
        Return a boolean indicating whether box should be displayed
        for specific object
        """
        object = self.REQUEST.get('object', None)
        if not object:
            zLOG.LOG(self.meta_type, zLOG.INFO, 'Object not found in request object. Request object contains:\n%s' % str(self.REQUEST))
            return 1
        showType = self.getShowupFor()
        objectIsObject = hasattr(object, 'SearchableText')
        showup = 0
        if showType == 'both':
            showup = 1
        if showType == 'object' and objectIsObject:
            showup = 1
        if showType == 'folder' and not objectIsObject:
            showup = 1
        return showup


    #security.declarePublic('isPortalTypeSelected')
    #def isPortalTypeSelected(self, typeid):
    #    """
    #    Checks whether the given typeid is present in the typesdict
    #    """
    #    if self.types_dict.has_key(typeid):
    #        return 1
    #    else:
    #        return 0

    #security.declarePublic('canDelete')
    #def canDelete(self):
    #    """
    #    Returns a boolean indicating whether you can delete the box object
    #    """
    #    can_del = 1
    #    if hasattr(self, 'can_delete'):
    #        if self.can_delete == 'true':
    #            can_del = 1
    #        else:
    #            can_del = 0
    #    return can_del

    #security.declarePublic('isFormatObjectSelected')
    #def isFormatObjectSelected(self, path):
    #    """
    #    Checks whether the given path to an format object is used for
    #    formatting a specific portal type
    #    """
    #    used = 0
    #    for key in self.types_dict.keys():
    #        if self.types_dict[key] == path:
    #            used = 1
    #    return used

    def manage_afterAdd(self, item, container):
        """ set the sort order """
        zLOG.LOG('MenuBase', zLOG.INFO, 'manage_afterAdd()')
        sort_order = 0
        for e in container.objectValues():
            #zLOG.LOG('AbstractElement', zLOG.INFO, 'manage_afterAdd: elem %s' % (e.getId()))
            try:
                if e.getId() != item.getId() and e.sort_order >= sort_order:
                    sort_order = (e.sort_order + 1)
            except:
                zLOG.LOG(self.meta_type, zLOG.INFO, 'manage_afterAdd: exception %s - %s\n%s' % (sys.exc_type, sys.exc_value, traceback.format_list(traceback.extract_tb(sys.exc_traceback))))

        self.sort_order = sort_order

