##############################################################################
#    Copyright (C) 2002-03  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import os, zLOG, random, sys, re
from Globals import InitializeClass, package_home
from Persistence import Persistent
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base, aq_parent, aq_inner
from OFS.Folder import Folder
from OFS.SimpleItem import SimpleItem
from Products.PageTemplates import PageTemplateFile
from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.Expression import Expression
from DateTime import DateTime
from FormatObject import FormatObject
from utils import getUniqueId

_zptdir = os.path.join(package_home(globals()), 'www')

webcolors = ('aqua','black','blue','fuchsia','gray','green','lime','maroon','navy','olive','purple','red','silver','teal','white','yellow')

class LayoutTool(UniqueObject, Folder, ActionProviderBase):

    """
    A tool for creating portal layouts
    """

    id = 'portal_layout'
    meta_type = 'Layout Tool'
    colors = webcolors
    _layout_types = []
    _available_types = []
    _page_layouts = {}
    _msgcatalog = 'cmf_portallayout'
    _maincolumn = None
    COLUMN_PREFIX = 'clf_'
    TOPBAR_PREFIX = 'clftop_'

    _format_elements = [ # default format element list - mostly Dublin Core
                 {"title":"Title","expression":"obj/title_or_id"}
                ,{"title":"Description","expression":"obj/Description"}
                ,{"title":"Subject","expression":"python:modules['string'].join(obj.Subject(),', ')"}
                ,{"title":"Creator","expression":"obj/Creator"}
                ,{"title":"Publisher","expression":"obj/Publisher"}
                ,{"title":"Contributors","expression":"python:strong.join(obj.Contributors(),', ')"}
                ,{"title":"Type","expression":"obj/Type"}
                ,{"title":"CreationDate","expression":"obj/CreationDate"}
                ,{"title":"ModificationDate","expression":"obj/ModificationDate"}
                ,{"title":"EffectiveDate","expression":"obj/EffectiveDate"}
                ,{"title":"ExpirationDate","expression":"obj/ExpirationDate"}
                ,{"title":"Format","expression":"obj/Format"}
                ,{"title":"Language","expression":"obj/Language"}
                ,{"title":"Rights","expression":"obj/Rights"}
                ,{"title":"Icon","expression":"python:obj.restrictedTraverse(obj.getIcon())"}
                #,NEWLINE:{"expression":""}
               ]

    _styles = [
               {"title":"Heading","font_size":"","font_weight":"bold","font_style":"","font_color":""}
              ,{"title":"LargeHeading","font_size":"120%","font_weight":"bold","font_style":"","font_color":""}
              ,{"title":"Description","font_size":"","font_weight":"","font_style":"italic","font_color":""}
              ]

    _links = [{'title': 'CreatorHome', 'expression': 'python:portal.portal_membership.getHomeUrl(obj.Creator())'}
                , {'title': 'ObjectHome', 'expression': 'obj/absolute_url'}
                 ]

    _actions = [ ActionInformation(
                    id='edit_layout'
                  , title='Layout'
                  , action=Expression(
                        text='string: ${folder_url}/edit_layout_columns')
                  , condition=''
                  , permissions=(CMFCorePermissions.ManagePortal,)
                  , category='folder'
                  , visible=1
                  )
                ,ActionInformation(
                    id='layout_types_edit'
                  , title='Types'
                  , action=Expression(
                        text='string: ${portal_url}/layouttypes_editform')
                  , condition=None
                  , permissions=(CMFCorePermissions.ManagePortal,)
                  , category='configuration'
                  , visible=1
                  )
                , ActionInformation(
                    id='layout_styles_edit'
                  , title='Styles'
                  , action=Expression(
                        text='string: ${portal_url}/layoutstyles_editform')
                  , condition=None
                  , permissions=(CMFCorePermissions.ManagePortal,)
                  , category='configuration'
                  , visible=1
                  )
                , ActionInformation(
                    id='layout_formats_edit'
                  , title='Formats'
                  , action=Expression(
                        text='string: ${portal_url}/layoutformats_editform')
                  , condition=None
                  , permissions=(CMFCorePermissions.ManagePortal,)
                  , category='configuration'
                  , visible=1
                  )
                , ActionInformation(
                   id='msgcatalog_edit'
                 , title='Message Catalog'
                 , action=Expression(text='string:${portal_url}/edit_layout_msgcatalog')
                 , condition=None
                 , permissions=(CMFCorePermissions.ManagePortal,)
                 , category='configuration'
                 , visible=1
                 )
                , ActionInformation(
                  id='sitestyles_edit'
                , title='Site Styles'
                , action=Expression(text='string:${portal_url}/sitestyles_edit_form')
                , condition=None
                , permissions=(CMFCorePermissions.ManagePortal,)
                , category='configuration'
                , visible=1
                )
               ]

    security = ClassSecurityInfo()

    manage_options = ( { 'label' : 'Overview', 'action' : 'manage_overview' }
                     , { 'label' : 'Types', 'action' : 'manage_Types' }
                     , { 'label' : 'Formats', 'action' : 'manage_Formats' }
                     , { 'label' : 'Format Elements', 'action' : 'manage_FormatElements' }
                     , { 'label' : 'Styles', 'action' : 'manage_Styles' }
                     , { 'label' : 'Links', 'action' : 'manage_Links' }
                     , { 'label' : 'Slots', 'action' : 'manage_Slots' }
                     ) + ActionProviderBase.manage_options + SimpleItem.manage_options

    #
    #   ZMI methods
    #
    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_overview' )
    manage_overview = PageTemplateFile.PageTemplateFile( 'explainLayoutTool', _zptdir)

    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_Types' )
    manage_Types = PageTemplateFile.PageTemplateFile( 'layouttoolSetTypes', _zptdir)

    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_Formats' )
    manage_Formats = PageTemplateFile.PageTemplateFile( 'layouttoolFormats', _zptdir)

    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_renameFormatsForm' )
    manage_renameFormatsForm = PageTemplateFile.PageTemplateFile( 'layouttoolRenameFormat_form', _zptdir)

    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_FormatElements' )
    manage_FormatElements = PageTemplateFile.PageTemplateFile( 'formatElementLines', _zptdir)

    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_Styles' )
    manage_Styles = PageTemplateFile.PageTemplateFile( 'formatStyles', _zptdir)

    security.declareProtected( CMFCorePermissions.ManagePortal
                            , 'manage_Links' )
    manage_Links = PageTemplateFile.PageTemplateFile( 'formatLinks', _zptdir)

    security.declareProtected( CMFCorePermissions.ManagePortal
                            , 'manage_Slots')
    manage_Slots = PageTemplateFile.PageTemplateFile( 'layoutSlots', _zptdir)


    #
    #   'portal_menutool' interface methods
    #

    #menuformat_types = [{'name':'Icon', 'type':'icon'}
    #                 , {'name':'Title', 'type':'title'}
    #                 , {'name':'Description', 'type':'description'}
    #                 , {'name':'Date', 'type':'date'}
    #                 , {'name':'Author', 'type': 'author'}]

    security.declareProtected(CMFCorePermissions.View, 'all_meta_types')
    def all_meta_types(self):
        """ satisfy copy support """
        return [{"name":"Format Object","action":"manage_addFormat"},]

    security.declarePublic('findAllLayoutObjects')
    def findAllLayoutObjects(self):
        """
        Return a list of all availiable layout objects
        """
        portal = getToolByName(self, 'portal_url').getPortalObject()
        findres = self.PrincipiaFind(portal, obj_metatypes=self.listLayoutTypes(), search_sub=1)
        res = []
        for findtuple in findres:
            parent = findtuple[0]
            obj = findtuple[1]
            res.append(obj)
        return res

    security.declarePublic('getSortByIndexes')
    def getSortByIndexes(self):
        """
        Return the ids of indexes which can be used as sort_on index
        """
        sort_on = []
        catalog = self.portal_catalog
        indexes = catalog._catalog.indexes
        for index in indexes.keys():
            if hasattr(indexes[index], 'keyForDocument'):
                sort_on.append(index)
        sort_on.sort()
        return sort_on

    security.declarePrivate('listActions')
    def listActions(self, info=None):
        """
        Return actions provided by tool
        """
        return self._actions

    security.declarePublic('getId')
    def getId(self):
        """
        Return the id
        """
        return self.id

    security.declarePublic('getMessageCatalog')
    def getMessageCatalog(self):
        """
        Return the name of the message catalog
        """
        return self._msgcatalog

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_setMessageCatalog')
    def manage_setMessageCatalog(self, REQUEST):
        """
        Set the name of the message catalog
        """
        req = REQUEST
        msgcatalog = req.get('msgcatalog', None)
        if msgcatalog is not None:
            self._msgcatalog = msgcatalog
            msg = 'Message Catalog updated'
        else:
            msg = 'No message catalog selected'

        req_url = req.get('HTTP_REFERER', None)
        if not req_url:
            portal = getToolByName(self, 'portal_url').getPortalObject()
            req_url = '%s/edit_layout_messagecatalog' % portal.absolute_url()
        has_q = req_url.find('?')
        if has_q != -1:
            req_url,trash = req_url.split('?')
        req.RESPONSE.redirect('%s?frameable=true&portal_status_message=%s' % (req_url, msg.replace(' ', '+')))

    security.declareProtected(CMFCorePermissions.ManagePortal, 'setLayoutMerging')
    def setLayoutMerging(self, container, state, slot):
        """ set a flag on whether we should merge parent layouts """
        layoutcontainer = self.getLayoutContainerFor(container, inner=1, create=1, slot=slot)
        setattr(layoutcontainer, 'merge_layouts', state)

    security.declarePublic('isMergingEnabledFor')
    def isMergingEnabledFor(self, container, slot):
        """
        returns a boolean value indicating whether layoutmerging is enabled
        """
        layoutcontainer = self.getLayoutContainerFor(container, inner=1, create=0, slot=slot)
        if layoutcontainer and hasattr(aq_base(layoutcontainer), 'merge_layouts'):
            return layoutcontainer.merge_layouts
        return 0

    security.declarePublic('getParentContainerFor')
    def getParentContainerFor(self, container):
        """
        Return the next container upwards in the hierarchy which
        could have a layoutcontainer.
        """
        if not getattr(container, '_isPortalRoot', None):
            item = container
            _notfound = []
            while 1:
                item = getattr(item, 'aq_parent', _notfound)
                if item is _notfound:
                    break
                elif item.isPrincipiaFolderish:
                    break
                elif item.id == 'talkback':
                    break
            if item is not container:
                #zLOG.LOG('MenuTool', zLOG.INFO, 'getParentContainerFor: %s != %s' % (container, item))
                return item

    security.declarePublic('listLayoutObjectsFor')
    def listLayoutObjectsFor(self, column, ctype='column'):
        """
        Returns all the layout item objects created in container.
        If layout merging is enabled any layouts from the parent container
        is appended to the list.
        Container must be layout container
        """
        if ctype == 'column':
            if column.getId()[:4] != self.COLUMN_PREFIX:
                column = self.getLayoutColumnsFor(column)
        elif ctype == 'topbar':
            if column.getId()[:7] != self.TOPBAR_PREFIX:
                column = self.getLayoutTopbarFor(column)
        else:
            raise AttributeError, 'Unknown column type: %s' % ctype

        layouts = []
        if column:
            if column.objectValues():
                layouts = list(column.objectValues(self.listLayoutTypes()))
                layouts.sort()
                """
                if self.isMergingEnabledFor(container) and not inner:
                    pcontainer = self.getParentContainerFor(container)
                    if pcontainer:
                        playouts = self.listLayoutItemsFor(pcontainer, slot=slot)
                        if playouts:
                            layouts = playouts + layouts
                #zLOG.LOG('MenuTool', zLOG.INFO, 'listMenuItemsFor: %s - returning: %s' % (container, str(menus)))
                """
        return layouts

    security.declarePublic('listLayoutObjectsForLanguage')
    def listLayoutObjectsForLanguage(self, container, ctype='column'):
        """
        Returns all layout items created in container
        Uses language to determine what items dhould be returned
        """
        layouts = self.listLayoutObjectsFor(container, ctype)
        try:
            msgcatalog = self.restrictedTraverse(self.getMessageCatalog())
            sel_language = msgcatalog.get_selected_language()
        except:
            sel_language = 'en'
        for layout in layouts:
            lan = layout.getLanguage()
            if lan and lan != sel_language:
                layouts.remove(layout)
        return layouts

    security.declareProtected(CMFCorePermissions.ManagePortal, 'deleteLayoutIn')
    def deleteLayoutIn(self, columnfolder, layout_id):
        """ Delete selected layout(s) """
        if type(layout_id) is type(''):
            columnfolder._delObject(layout_id)
        elif type(layout_id) in (type(()), type([])):
            for lid in layout_id:
                columnfolder._delObject(lid)

    security.declareProtected(CMFCorePermissions.ManagePortal, 'moveLayoutIn')
    def moveLayoutIn(self, columnfolder, layout_id, direction, ctype):
        """ Move selected layout(s) """
        #zLOG.LOG('MenuTool', zLOG.INFO, 'moveMenuIn: %s' % (container))
        swapobj = None

        swap1 = getattr(columnfolder, layout_id)
        #zLOG.LOG('MenuTool', zLOG.INFO, 'moveMenuIn - menu %s' % (swap1.title_or_id()))
        objs = self.listLayoutObjectsFor(columnfolder, ctype)

        #zLOG.LOG('MenuTool', zLOG.INFO, 'moveMenuIn - moving "%s"' % (direction))
        if direction == 'up':
            #zLOG.LOG('MenuTool', zLOG.INFO, 'moveMenuIn - moving "%s"' % (direction))
            for layout in objs:
                if swapobj is None: swapobj = layout
                if layout.getId() != id:
                    if layout.sort_order < swap1.sort_order and layout.sort_order > swapobj.sort_order:
                        swapobj = layout
            if swapobj is not None:
                tmp = swapobj.sort_order
                swapobj.setSortOrder(swap1.sort_order)
                swap1.setSortOrder(tmp)
        elif direction == 'down':
            #zLOG.LOG('MenuTool', zLOG.INFO, 'moveMenuIn - moving "%s"' % (direction))
            for layout in objs:
                if swapobj is None: swapobj = layout
                if layout.getId() != id:
                    if layout.sort_order > swap1.sort_order and layout.sort_order > swapobj.sort_order:
                        swapobj = layout
                        break
            if swapobj is not None:
                tmp = swapobj.sort_order
                swapobj.setSortOrder(swap1.sort_order)
                swap1.setSortOrder(tmp)

    security.declareProtected(CMFCorePermissions.ManageProperties, 'deleteLayoutTopbarFor')
    def deleteLayoutTopbarFor(self, topbarfolder):
        """
        Delete the requested top bar
        """
        if type(topbarfolder) is type(''):
            topbarfolder = self.portal_url.getPortalObject().restrictedTraverse(topbarfolder)
        container = self.getParentContainerFor(topbarfolder)
        if container and container.getId() != topbarfolder.getId():
            container._delObject(topbarfolder.getId())

    security.declareProtected(CMFCorePermissions.ManageProperties, 'addLayoutTopbarIn')
    def addLayoutTopbarIn(self, container):
        """
        Add a new layout topbar in 'container'
        """
        if type(container) is type(''):
            container = self.portal_url.getPortalObject().restrictedTraverse(container)
        columnid = "%s%s" % (self.TOPBAR_PREFIX, getUniqueId())
        container.manage_addProduct['OFSP'].manage_addFolder(columnid,title="CMF Portal Layout Topbar")

    security.declarePublic('getLayoutTopbarFor')
    def getLayoutTopbarFor(self, container, aquire=1):
        """
        Return the layout topbar
        """
        topbar = None
        portal = getToolByName(self, 'portal_url').getPortalObject()
        if type(container) is type(''):
            container = portal.restrictedTraverse(container)
        if not container.isPrincipiaFolderish:
            container = aq_parent(container)
        containerid = container.getId()
        if containerid[:7] == self.TOPBAR_PREFIX:
            topbar = container
        else:
            zLOG.LOG('Layout Tool - get topbar', zLOG.INFO, 'Container is type %s' % container.meta_type)
            if container.id == 'talkback':
                container = self.getParentContainerFor(container)
            for ob in container.objectValues(['Folder']):
                if ob.getId()[:7] == self.TOPBAR_PREFIX:
                    topbar = ob
                    break
        if not topbar and container != portal and aquire:
            parent = container.aq_parent
            topbar = self.getLayoutTopbarFor(parent)
        if topbar:
            return topbar

    security.declarePublic('deleteLayoutContainerFor')
    def deleteLayoutColumnFor(self, columnfolder):
        """
        Delete the request column folder
        """
        if type(columnfolder) is type(''):
            columnfolder = self.portal_url.getPortalObject().restrictedTraverse(columnfolder)
        container = self.getParentContainerFor(columnfolder)
        if container and container.getId() != columnfolder.getId():
            columnpaths = container.getProperty('columnList')
            portal_url = getToolByName(self, 'portal_url')
            columnpaths.remove(portal_url.getRelativeContentPath(columnfolder))
            container._delObject(columnfolder.getId())
        container.manage_changeProperties(columnList=columnpaths)

    security.declareProtected(CMFCorePermissions.ManagePortal, 'addLayoutColumnIn')
    def addLayoutColumnIn(self, container):
        """
        Add a new layout column in 'container'
        """
        if type(container) is type(''):
            container = self.portal_url.getPortalObject().restrictedTraverse(container)
        columnid = "%s%s" % (self.COLUMN_PREFIX, getUniqueId())
        container.manage_addProduct['OFSP'].manage_addFolder(columnid,title="CMF Portal Layouts")
        column = getattr(container, columnid)
        if not container.hasProperty('columnList'):
            container.manage_addProperty(id='columnList', value=[], type='lines')
        columns = container.getProperty('columnList')
        portal_url = getToolByName(self, 'portal_url')
        newpath = portal_url.getRelativeContentPath(column)
        columns.append(newpath)
        container.manage_changeProperties(columnList=columns)

    security.declarePublic('getLayoutColumnsFor')
    def getLayoutColumnsFor(self, container, aquire=1):
        """
        Return the layout containers
        """
        columns = []
        if type(container) is type(''):
            container = self.portal_url.getPortalObject().restrictedTraverse(container)
        zLOG.LOG('Layout Tool - get layout column', zLOG.INFO, 'Container is %s and path %s' % \
            (str(container), container.absolute_url()))
        if not container.isPrincipiaFolderish:
            zLOG.LOG('Layout Tool - get layout column', zLOG.INFO, 'Container is not folderish')
            container = aq_parent(container)
            zLOG.LOG('Layout Tool - get layout column', zLOG.INFO, 'Aqcuired container is %s and path %s' % \
            (str(container), container.absolute_url()))
        containerid = container.getId()
        if containerid[:4] == self.COLUMN_PREFIX:
            container = self.getParentContainerFor(container)
        columnpaths = container.getProperty('columnList', [])
        zLOG.LOG('Layout Tool - get layout column', zLOG.INFO, 'Column path is %s' % str(columnpaths))
        portal = getToolByName(self, 'portal_url').getPortalObject()
        for path in columnpaths:
            column = portal.restrictedTraverse(path)
            if column.getId()[:4] == self.COLUMN_PREFIX:
                columns.append(column)
        portal = getToolByName(self, 'portal_url').getPortalObject()
        if not columns and container != portal and aquire:
            parent = container.aq_parent
            columns = self.getLayoutColumnsFor(parent)
        if columns:
            return columns
        """
        else:
            zLOG.LOG('Layout Tool - get columns', zLOG.INFO, "We still don't have any columns")
        """

    security.declareProtected(CMFCorePermissions.ManagePortal, 'moveLayoutColumn')
    def moveLayoutColumn(self, container, cidx, direction):
        """
        Move the requested column in direction
        """
        cidx = int(cidx)
        paths = container.getProperty('columnList')
        if direction == 'up':
            if cidx > 0 and cidx < len(paths):
                old = paths[cidx - 1]
                paths[cidx - 1] = paths[cidx]
                paths[cidx] = old
        elif direction == 'down':
            if cidx < len(paths)-1 and cidx >= 0:
                old = paths[cidx + 1]
                paths[cidx + 1] = paths[cidx]
                paths[cidx] = old
        else:
            raise AttributeError, 'Unknown direction: %s' % direction
        container.manage_changeProperties(columnList=paths)

    security.declarePublic('deleteLocalLayoutColumnFor')
    def deleteLocalLayoutColumnFor(self, columnfolder):
        """
        Delete the request column folder
        """
        if type(columnfolder) is type(''):
            columnfolder = self.portal_url.getPortalObject().restrictedTraverse(columnfolder)
        container = self.getParentContainerFor(columnfolder)
        if container and container.getId() != columnfolder.getId():
            columnpaths = container.getProperty('localColumnList')
            portal_url = getToolByName(self, 'portal_url')
            columnpaths.remove(portal_url.getRelativeContentPath(columnfolder))
            container._delObject(columnfolder.getId())
        container.manage_changeProperties(localColumnList=columnpaths)

    security.declareProtected(CMFCorePermissions.ManagePortal, 'addLocalLayoutColumnIn')
    def addLocalLayoutColumnIn(self, container):
        """
        Add a new layout column in 'container'
        """
        if type(container) is type(''):
            container = self.portal_url.getPortalObject().restrictedTraverse(container)
        columnid = "%s%s" % (self.COLUMN_PREFIX, getUniqueId())
        container.manage_addProduct['OFSP'].manage_addFolder(columnid,title="CMF Portal Layouts")
        column = getattr(container, columnid)
        if not container.hasProperty('localColumnList'):
            container.manage_addProperty(id='localColumnList', value=[], type='lines')
        columns = container.getProperty('localColumnList')
        portal_url = getToolByName(self, 'portal_url')
        newpath = portal_url.getRelativeContentPath(column)
        columns.append(newpath)
        container.manage_changeProperties(localColumnList=columns)

    security.declarePublic('getLocalLayoutColumnsFor')
    def getLocalLayoutColumnsFor(self, container, aquire=1):
        """
        Return the layout containers
        """
        columns = []
        if type(container) is type(''):
            container = self.portal_url.getPortalObject().restrictedTraverse(container)
        if not container.isPrincipiaFolderish:
            container = aq_parent(container)
        containerid = container.getId()
        if containerid[:4] == self.COLUMN_PREFIX:
            container = self.getParentContainerFor(container)
        columnpaths = container.getProperty('localColumnList', [])
        portal = getToolByName(self, 'portal_url').getPortalObject()
        for path in columnpaths:
            column = portal.restrictedTraverse(path)
            if column.getId()[:4] == self.COLUMN_PREFIX:
                columns.append(column)
        portal = getToolByName(self, 'portal_url').getPortalObject()
        if not columns and container != portal and aquire:
            parent = container.aq_parent
            columns = self.getLocalLayoutColumnsFor(parent)
        if columns:
            return columns
        """
        else:
            zLOG.LOG('Layout Tool - get columns', zLOG.INFO, "We still don't have any columns")
        """

    security.declareProtected(CMFCorePermissions.ManagePortal, 'moveLocalLayoutColumn')
    def moveLocalLayoutColumn(self, container, cidx, direction):
        """
        Move the requested column in direction
        """
        cidx = int(cidx)
        paths = container.getProperty('localColumnList')
        if direction == 'up':
            if cidx > 0 and cidx < len(paths):
                old = paths[cidx - 1]
                paths[cidx - 1] = paths[cidx]
                paths[cidx] = old
        elif direction == 'down':
            if cidx < len(paths)-1 and cidx >= 0:
                old = paths[cidx + 1]
                paths[cidx + 1] = paths[cidx]
                paths[cidx] = old
        else:
            raise AttributeError, 'Unknown direction: %s' % direction
        container.manage_changeProperties(localColumnList=paths)

    security.declarePublic('getLayoutPropertiesFor')
    def getLayoutPropertiesFor(self, container, aquire=1):
        """
        Return the layout properties for the page design
        """
        props = None
        if not container.isPrincipiaFolderish:
            container = container.aq_parent
        path = container.absolute_url(relative=1)
        #zLOG.LOG('Layout tool - get layout props', zLOG.INFO, 'Path is %s' % path)
        pagelayouts = self._page_layouts
        #zLOG.LOG('Layout tool - get layout props', zLOG.INFO, 'Layouts is %s' % str(pagelayouts))
        if pagelayouts.has_key(path):
            #zLOG.LOG('Layout tool - get layout props', zLOG.INFO, 'layout is %s' % str(pagelayouts[path]))
            props = pagelayouts[path]
        portal = getToolByName(self, 'portal_url').getPortalObject()
        if not props and container != portal and aquire:
            container = container.aq_parent
            props = self.getLayoutPropertiesFor(container)
        return props

    security.declareProtected(CMFCorePermissions.ManagePortal, 'setLayoutPropertiesFor')
    def setLayoutPropertiesFor(self, container, width, align, spacing, padding):
        """
        Set the layout properties for the page layout
        """
        pagelayouts = self._page_layouts
        path = container.absolute_url(relative=1)
        props = {'width': width, 'align': align, 'cellspacing': spacing, 'cellpadding': padding}
        pagelayouts[path] = props
        self._page_layouts = pagelayouts
        self._p_changed = 1

    security.declarePublic('getMainColumn')
    def getMainColumn(self):
        """
        Return the main column (the one containing metal:define-slot="main")
        """
        column = None
        if self._maincolumn:
            portal = getToolByName(self, 'portal_url').getPortalObject()
            column = portal.restrictedTraverse(self._maincolumn)
        return column

    security.declareProtected(CMFCorePermissions.ManagePortal, 'setMainColumn')
    def setMainColumn(self, cpath):
        """
        Set the path to the main column
        """
        self._maincolumn = cpath

    security.declareProtected(CMFCorePermissions.ManageProperties, 'addLayoutTo')
    def addLayoutTo(self, columnfolder, layout_type, title=None):
        if type(columnfolder) is type(''):
            columnfolder = self.portal_url.getPortalObject().restrictedTraverse(columnfolder)
        layout = None
        #zLOG.LOG('Layout tool - add layout', zLOG.INFO, 'Columnfolder path: %s' % columnfolder.absolute_url())
        for lt in self._layout_types:
            if layout_type == lt.meta_type:
                layout = lt(title)
                break

        if layout is None:
            # No menu type matched passed in value
            raise NameError, 'Unknown layout Type: %s' % type
        try:
            columnfolder._setObject(layout.getId(), layout)
        except:
            zLOG.LOG('LayoutTool', zLOG.INFO, 'Exception in addLayoutTo: Type: %s - Value: %s' % \
                   (sys.exc_info()[0], sys.exc_info()[1]))

    security.declarePublic('listLayoutTypes')
    def listLayoutTypes(self):
        """
        Returns a list of meta_types
        """
        types = []
        for type in self._layout_types:
            types.append(type.meta_type)
        #zLOG.LOG('LayoutTool', zLOG.INFO, 'listLayoutTypes: %s' % str(types))
        return types
        
    security.declarePublic('listLayoutClasses')
    def listLayoutClasses(self):
        """
        Returns a list of available layout classes
        """
        return self._layout_types

    security.declarePublic('listAvailableTypes')
    def listAvailableTypes(self):
        """
        Returns a list of portal types available for list menus
        """
        #zLOG.LOG('LayoutTool', zLOG.INFO, 'listAvailableTypes: %s' % str(self._available_types))
        avail_types = []
        types = self.portal_types.listContentTypes()
        for ltype in self._available_types:
            if ltype in types:
                avail_types.append(ltype)
        return avail_types

    security.declarePublic('listFormats')
    def listFormats(self):
        """
        Returns a list of formats
        """
        return self.objectValues('Format Object')

    security.declarePublic('listLinkElements')
    def listLinkElements(self):
        """
        Returns a list of link elements
        """
        return self._links

    security.declarePublic('listFormatElements')
    def listFormatElements(self):
        """
        Returns a list of format elements
        """
        return self._format_elements

    security.declarePublic('getFormat')
    def getFormat(self, id):
        """
        Returns the formats object with id 'id'
        """
        try:
            obj = self._getOb(id)
            return obj
        except:
            pass

    security.declarePublic('listStyles')
    def listStyles(self):
        """
        Returns the style list
        """
        return self._styles

    security.declarePublic('getStyleByName')
    def getStyleByName(self, name):
        """
        Returns the style dictionary with title 'name' or None if not found
        """
        for style in self._styles:
            if style['title'] == name:
                return style

    security.declarePublic('getStyleLineByName')
    def getStyleLineByName(self, name):
        """
        Returns a css-formatted line compiled from style name
        """
        style = self.getStyleByName(name)
        if style:
            line = ''
            if style.get('font_size', None):
                line = "%s font-size: %s;" % (line, style['font_size'])
            if style.get('font_style', None):
                line = "%s font-style: %s;" % (line, style['font_style'])
            if style.get('font_weight', None):
                line = "%s font-weight: %s;" % (line, style['font_weight'])
            if style.get('font_color', None):
                line = "%s color: %s;" % (line, style['font_color'])

            return line

    security.declarePrivate('_testStyleTitleValidity')
    def _testStyleTitleValidity(self, title, index=None):
        """
        Test if the title is already occupied
        """
        if not title:
            raise "ValueError", "An empty title is not allowed"
        idx = 0
        for style in self._styles:
            if style['title'] == title:
                if index is None or index != idx:
                    raise "ValueError", "The title '%s' is already in use" % title
            idx = idx + 1

    security.declarePrivate( '_extractStyle' )
    def _extractStyle( self, properties, index ):

        """
        Extract a style from the funky form properties.
        Got this from ActionProviderBase.
        """
        title       = str( properties.get( 'title_%d'       % index, '' ) )
        font_size   = str( properties.get( 'font_size_%d'   % index, '' ) )
        font_style  = str( properties.get( 'font_style_%d'  % index, '' ) )
        font_weight = str( properties.get( 'font_weight_%d' % index, '' ) )
        font_color  = str( properties.get( 'font_color_%d'  % index, '' ) )

        rgx = re.compile(r'(^_|[^a-zA-Z0-9-_~\,\.])')
        title = re.sub(rgx, "",title)
        self._testStyleTitleValidity(title, index)
        style = {
                "title":title
               ,"font_size":font_size
               ,"font_style":font_style
               ,"font_weight":font_weight
               ,"font_color":font_color
               }

        return style

    security.declarePrivate('_testSlotIdValidity')
    def _testSlotIdValidity(self, slotid, index=None):
        """
        Test if the slotid is already occupied
        """
        if not slotid:
            raise "ValueError", "An empty slot id is not allowed"
        idx = 0
        for slot in self._slots:
            if slot == slotid:
                if index is None or index != idx:
                    raise "ValueError", "The slot id '%s' is already in use" % slotid
            idx = idx + 1

    security.declarePublic('getFormatElementExpression')
    def getFormatElementExpression(self, name):
        """
        Returns the TALES expression for 'name'
        """
        for element in self._format_elements:
            if element['title'] == name:
                return element['expression']

    security.declarePublic('getLinkExpression')
    def getLinkExpression(self, name):
        """
        Return the TALES expression for 'name'
        """
        for element in self._links:
            if element['title'] == name:
                return element['expression']
        return None

    security.declarePrivate( '_extractFormatElement' )
    def _extractFormatElement( self, properties, index ):

        """
        Extract a style from the funky form properties.
        Got this from ActionProviderBase.
        """
        title       = str( properties.get( 'title_%d'       % index, '' ) )
        expression  = str( properties.get( 'expression_%d'   % index, '' ) )

        format_element = {
                "title":title
               ,"expression":expression
               }

        return format_element

    security.declarePrivate('_extractLinkElement')
    def _extractLinkElement(self, properties, index):
        """
        Extract the link from the properties
        """
        title = str(properties.get('title_%d' % index, ''))
        expression = str(properties.get('expression_%d' % index, ''))
        link_element = {'title': title, 'expression': expression}
        return link_element

    # ZMI methods
    # format objects
    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_setTypes')
    def manage_setTypes(self, current_types, noredirect=0):
        """
        Change the types available for list menus
        """
        #zLOG.LOG('MenuTool', zLOG.INFO, 'manage_setTypes: %s' % str(current_types))
        self._available_types = current_types
        if not noredirect:
            return self.manage_Types(
                self, self.REQUEST, manage_tabs_message='Properties changed.')

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_addFormat')
    def manage_addFormat(self, id='', title='', REQUEST=None):
        """
        Change the types available for list menus
        """
        format = FormatObject(id, title)
        self._setObject(format.getId(), format)

        if REQUEST is not None:
            return self.manage_Formats(
                self, self.REQUEST, manage_tabs_message='Format added.')

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_renameFormat')
    def manage_renameFormat(self, REQUEST):
        """
        Rename a list of formats
        """
        self.manage_renameObjects(REQUEST['ids'], REQUEST['new_ids'], REQUEST)
        return self.manage_Formats(
            self, self.REQUEST, manage_tabs_message='Format renamed.')

    security.declareProtected(CMFCorePermissions.ManagePortal, 'manage_deleteFormat')
    def manage_deleteFormat(self, ids=(), REQUEST=None):
        """
        Delete select format object(s)
        """
        if type(ids) == type(()) or type(ids) == type([]):
            for id in ids:
                self._delObject(id)
        else:
            self._delObject(ids)

        if REQUEST is not None:
            return self.manage_Formats(
                self, self.REQUEST, manage_tabs_message='Format(s) deleted.')

    # Styles
    security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_editStyles' )
    def manage_editStyles( self, properties=None, REQUEST=None ):

        """ Update our list of styles.
        """
        if properties is None:
            properties = REQUEST

        styles = []

        for index in range( len( self._styles ) ):
            styles.append( self._extractStyle( properties, index ) )

        self._styles = tuple( styles )

        if REQUEST is not None:
            return self.manage_Styles(
                self, self.REQUEST, manage_tabs_message='Saved.')

    security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_addStyle')
    def manage_addStyle(self,title,  REQUEST=None):
        """
        Add a new style dictionary.
        """
        rgx = re.compile(r'(^_|[^a-zA-Z0-9-_~\,\.])')
        title = re.sub(rgx, "",title)
        self._testStyleTitleValidity(title)

        style = {
                "title":title
               ,"font_size":''
               ,"font_style":''
               ,"font_weight":''
               ,"font_color":''
               }
        styles = list(self._styles)
        styles.append(style)
        self._styles = tuple(styles)
        self._p_changed = 1

        if REQUEST is not None:
            return self.manage_Styles(
                self, self.REQUEST, manage_tabs_message='%s added.' % title)

    security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_deleteStyles')
    def manage_deleteStyles(self, ids=(), REQUEST=None):
        """
        Delete the requested formatting
        """
        ids = list( map( int, ids ) )  # Convert to a list of integers.
        if ids:
            new_styles = []
            styles = list(self._styles)
            for idx in range(len(styles)):
                if idx not in ids:
                    new_styles.append(styles[idx])
            self._styles = tuple(new_styles)
            self._p_changed = 1
        if REQUEST:
            return self.manage_Styles(
                self, self.REQUEST, manage_tabs_message='Style(s) deleted.')

    # format elements
    security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_editFormatElements' )
    def manage_editFormatElements( self, properties=None, REQUEST=None ):

        """ Update our list of format elements.
        """
        if properties is None:
            properties = REQUEST

        format_elements = []

        for index in range( len( self._format_elements ) ):
            format_elements.append( self._extractFormatElement( properties, index ) )

        self._format_elements = tuple( format_elements )

        if REQUEST is not None:
            return self.manage_FormatElements(
                self, self.REQUEST, manage_tabs_message='Saved.')

    security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_addFormatElement')
    def manage_addFormatElement(self,title, expression, REQUEST=None):
        """
        Add a new format element dictionary.
        """
        format_element = {
                "title":title
               ,"expression":expression
               }
        format_elements = list(self._format_elements)
        format_elements.append(format_element)
        self._format_elements = tuple(format_elements)
        self._p_changed = 1

        if REQUEST is not None:
            return self.manage_FormatElements(
                self, self.REQUEST, manage_tabs_message='%s added.' % title)

    security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_deleteFormatElements')
    def manage_deleteFormatElements(self, ids=(), REQUEST=None):
        """
        Delete the requested format element
        """
        ids = list( map( int, ids ) )  # Convert to a list of integers.
        if ids:
            new_format_elements = []
            format_elements = list(self._format_elements)
            for idx in range(len(format_elements)):
                if idx not in ids:
                    new_format_elements.append(format_elements[idx])
            self._format_elements = tuple(new_format_elements)
            self._p_changed = 1
        if REQUEST:
            return self.manage_FormatElements(
                self, self.REQUEST, manage_tabs_message='Format element(s) deleted.')
                
    # link elements
    security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_editLinks' )
    def manage_editLinks( self, properties=None, REQUEST=None ):

        """ Update our list of format elements.
        """
        if properties is None:
            properties = REQUEST

        link_elements = []

        for index in range( len( self._links) ):
            link_elements.append( self._extractLinkElement( properties, index ) )

        self._links = tuple( link_elements )

        if REQUEST is not None:
            return self.manage_Links(
                self, self.REQUEST, manage_tabs_message='Saved.')

    security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_addLink')
    def manage_addLink(self,title, expression, REQUEST=None):
        """
        Add a new link dictionary.
        """
        link_element = {"title":title, "expression":expression}
        link_elements = list(self._links)
        link_elements.append(link_element)
        self._links = tuple(link_elements)
        self._p_changed = 1

        if REQUEST is not None:
            return self.manage_Links(
                self, self.REQUEST, manage_tabs_message='%s added.' % title)

    security.declareProtected( CMFCorePermissions.ManagePortal, 'manage_deleteLinks')
    def manage_deleteLinks(self, ids=(), REQUEST=None):
        """
        Delete the requested Llink
        """
        ids = list( map( int, ids ) )  # Convert to a list of integers.
        if ids:
            new_link_elements = []
            link_elements = list(self._links)
            for idx in range(len(link_elements)):
                if idx not in ids:
                    new_link_elements.append(link_elements[idx])
            self._links = tuple(new_link_elements)
            self._p_changed = 1
        if REQUEST:
            return self.manage_Links(
                self, self.REQUEST, manage_tabs_message='Link(s) deleted.')

def registerLayoutType(type):
    """
    Registers a new layout type with the tool
    """
    #zLOG.LOG('registerLayoutType', zLOG.INFO, 'type: %s' % str(type))
    if not type in LayoutTool._layout_types:
        LayoutTool._layout_types.append(type)
    #zLOG.LOG('registerLayoutType', zLOG.INFO, 'types: %s' % str(LayoutTool._layout_types))

InitializeClass( LayoutTool )


#Depricated methods
"""
    security.declarePublic('getLayoutContainerFor')
    def getLayoutContainerFor(self, container, slot, create=0, inner=0):
        Returns the layout container for container.
        if slot:
            layout_folder_id = "%s%s" % (self.LAYOUT_PREFIX, slot)
        else:
            raise 'Attribute Error', 'Slot is not defined'
        if type(container) is type(''):
            container = self.portal_url.getPortalObject().restrictedTraverse(container)
        if not container.isPrincipiaFolderish:
            container = aq_parent(container)
        if container and container.getId() != layout_folder_id:
            if not hasattr(aq_base(container), layout_folder_id) and create:
                container.manage_addProduct['OFSP'].manage_addFolder( layout_folder_id,title="CMF Portal Layouts")
            try:
                if inner:
                    layouts = getattr(aq_base(container), layout_folder_id)
                else:
                    layouts = getattr(container, layout_folder_id)
            except:
                #zLOG.LOG('LayoutTool', zLOG.INFO, 'Exception in getLayoutContainer: Type: %s - Value: %s' % \
                 #  (sys.exc_info()[0], sys.exc_info()[1]))
                layouts = None
            return layouts
        else:
            return None
"""
