##############################################################################
#    Copyright (C) 2002  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import os, zLOG, sys
from Globals import InitializeClass, package_home
from AccessControl import getSecurityManager, ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions
from DateTime import DateTime
from Products.CMFPortalLayout.LayoutBase import LayoutBase
from Products.CMFPortalLayout.LayoutTool import registerLayoutType
from Products.CMFPortalLayout.utils import getUniqueId

class ActionsMenu(LayoutBase):
    """
    A menu containing portal actions
    """
    _provider = 'folder'
    _category = None
    _style = None
    _edit_method = 'actionsmenu_edit_form'
    _view_method = 'actionsmenu_view'

    security = ClassSecurityInfo()
    type = 'Actions Menu'
    meta_type = 'Actions Menu'

    def __init__(self, title=''):
        """
        Create a new actions menu object.
        """
        self.id = getUniqueId()
        self.setTitle(title)

    security.declareProtected(CMFCorePermissions.ManageProperties, 'edit')
    def edit(self, REQUEST):
        """
        Set the menus properties
        """
        #zLOG.LOG('ListMenu', zLOG.INFO, 'edit: REQUEST: %s' % (str(REQUEST)))
        self.setTitle(REQUEST.get('title', None))
        self.setCategory(REQUEST.get('category', None))
        self.setStyle(REQUEST.get('style', None))
        self.setProvider(REQUEST.get('provider', None))

        parent = self.getParent()
        msg = 'Saved'
        msg = 'Actions Menu Saved'
        req = REQUEST
        returnurl = req.get('returnToUrl', None)
        if not returnurl:
            returnurl = "%s/edit_layout_columns" % parent.absolute_url()
        req.RESPONSE.redirect('%s?title=%s&render_title=%s&sel_portal_type=%s&item_language=%s&edit_layoutob=%s' % \
                ( returnurl
                , req.get('title', '')
                , req.get('render_title', '')
                , req.get('sel_portal_type','')
                , req.get('item_language','')
                , req.get('edit_layoutob', '')))

    security.declarePublic('getCategory')
    def getCategory(self):
        """
        Return the name of the category the menu should render.
        """
        return self._category

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setCategory')
    def setCategory(self, category=None):
        """
        Set the id of the object, the box should contain.
        """
        if category is not None:
            self._category = category

    security.declarePublic('getStyle')
    def getStyle(self):
        """
        Return the name style in which the menu entries should be rendered.
        """
        return self._style

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setStyle')
    def setStyle(self, style=None):
        """
        Set the name of the style in which the menu entries should be rendered.
        """
        if style is not None:
            self._style = style

    security.declarePublic('getProvider')
    def getProvider(self):
        """
        Return the provider of the actions is the object itself or its folder..
        """
        if not hasattr(self, '_provider'):
            self._provider = 'folder'
        return self._provider

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setProvider')
    def setProvider(self, provider=None):
        """
        Set whether the provider of the actions is the object itself or its folder..
        """
        if provider is not None:
            self._provider = provider

    security.declarePublic('listMenuActions')
    def listMenuActions(self, content):
        """
        Return the object the box should contain.
        """
        #zLOG.LOG('ActionsMenu', zLOG.INFO, 'listActions, content is %s' % str(content))
        if not self._category:
            return None
        if self.getProvider()  == 'folder' and not content.isPrincipiaFolderish:
            content = content.aq_parent
        actionstool = getToolByName(self, 'portal_actions')
        actions = actionstool.listFilteredActionsFor(content)
        c_actions = []
        try:
            #zLOG.LOG(self.meta_type, zLOG.INFO, "listActions '%s'" % str(self._category))
            c_actions = actions[self._category]
        except:
            zLOG.LOG(self.meta_type, zLOG.INFO, "Exception in listMenuActions: %s - %s" % \
                (sys.exc_info()[0],sys.exc_info()[1]))
        #zLOG.LOG(self.meta_type, zLOG.INFO, "Actions %s" % str(c_actions))
        return c_actions

registerLayoutType(ActionsMenu)
