##############################################################################
#    Copyright (C) 2002  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import os, zLOG, sys, traceback, string
from Globals import InitializeClass, package_home
from AccessControl import getSecurityManager, ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions
from DateTime import DateTime
from Products.CMFPortalLayout.LayoutBase import LayoutBase
from Products.CMFPortalLayout.LayoutTool import registerLayoutType
from Products.CMFPortalLayout.utils import getUniqueId

class BreadcrumbsBox(LayoutBase):
    """
    A menu containing breadcrumbs
    """
    _contentpath = None
    _edit_method = 'breadcrumbsbox_edit_form'
    _view_method = 'breadcrumbsbox_view'
    _include_root = 1
    _addToFavorites = 1
    _style = None

    security = ClassSecurityInfo()
    type = 'Breadcrumbs Box'
    meta_type = 'Breadcrumbs Box'

    def __init__(self, title=''):
        """
        Create a new breadcrumbs object.
        """
        self.id = getUniqueId()
        self.setTitle(title)

    security.declareProtected(CMFCorePermissions.ManageProperties, 'edit')
    def edit(self, REQUEST):
        """
        Set the menus properties
        """
        self.setTitle(REQUEST.get('title', None))
        self.setIncludeRoot(REQUEST.get('include_root', None))
        self.setAddToFavorites(REQUEST.get('addto_favorites', None))
        self.setStyle(REQUEST.get('style', None))

        parent = self.getParent()
        msg = 'Saved'
        req = REQUEST
        returnurl = req.get('returnToUrl', None)
        if not returnurl:
            returnurl = "%s/edit_layout_columns" % parent.absolute_url()
        req.RESPONSE.redirect('%s?title=%s&edit_layoutob=%s&include_root=%s&addto_favorites=%s' % \
                ( returnurl
                , req.get('title', '')
                , req.get('edit_layoutob', '')
                , req.get('include_root', None)
                , req.get('addto_favorites', None)))
                
    security.declarePublic('getStyle')
    def getStyle(self):
        """
        Return the name style in which the menu entries should be rendered.
        """
        return self._style

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setStyle')
    def setStyle(self, style=None):
        """
        Set the name of the style in which the menu entries should be rendered.
        """
        if style is not None:
            self._style = style

    security.declarePrivate('setIncludeRoot')
    def setIncludeRoot(self, include_root=None):
        """
        Decide whether we should include link to portal root or not
        """
        if include_root is not None:
            self._include_root = 1
        else:
            self._include_root = 0

    security.declarePublic('includeRoot')
    def includeRoot(self):
        """
        Should we include link to portal root ?
        """
        return int(self._include_root)

    security.declarePrivate('setAddToFavorites')
    def setAddToFavorites(self, addto=None):
        """
        Decide whether or not there should be a link in the breadcrumbs
        for the addToFavorites script
        """
        if addto is not None:
            self._addToFavorites = 1
        else:
            self._addToFavorites = 0

    security.declarePublic('addToFavorites')
    def addToFavorites(self):
        """
        Should we add a link to the addtoFavorites script ?
        """
        return int(self._addToFavorites)

    security.declarePublic('getBreadcrumbs')
    def getBreadcrumbs(self, content):
        """
        Return the breadcrumbs
        """
        result = []
        urltool = getToolByName(self, 'portal_url')
        portal_url = urltool
        obj = portal = getToolByName(self, 'portal_url').getPortalObject()
        include_root = self.includeRoot()

        if type(content) is type(''):
            content = portal.restrictedTraverse(content)

        if include_root:
            result.append({'id': 'Home'
                                 , 'title': self.truncText(portal.title_or_id(),15)
                                 , 'longtitle': portal.title_or_id()
                                 , 'url': portal_url}
                                 )

        relative = urltool.getRelativeContentPath(content)

        for i in range(len(relative)):
            try:
                now = relative[:i+1]
                obj = portal.restrictedTraverse(now)
                if not now[ -1 ] == 'talkback':
                    oid = now[ -1 ]
                    otitle = self.truncText(obj.title_or_id(),15)
                    olong = obj.title_or_id()
                    extras = string.join( now, '/' )
                    ourl = "%s/%s" % (portal.absolute_url(), extras)
                    result.append({'id': oid
                                         , 'title': otitle
                                         , 'longtitle': olong
                                         , 'url': ourl
                                         })
            except:
                zLOG.LOG('BC Box', zLOG.ERROR, 'Result error is %s - %s\n%s' % \
                            (sys.exc_info()[0]
                            , sys.exc_info()[1]
                            , traceback.format_list(traceback.extract_tb(sys.exc_traceback))
                            ))

        return result

    security.declarePrivate('truncText')
    def truncText(self, text, size):
        """
        Truncate the text string
        """
        if len(text) > size:
            return text[:size] + '...'
        else:
            return text

registerLayoutType(BreadcrumbsBox)
