##############################################################################
#    Copyright (C) 2002  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import os, zLOG, calendar, sys, traceback
from Globals import InitializeClass, package_home
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import _getViewFor
from DateTime import DateTime
from Products.CMFPortalLayout.LayoutBase import LayoutBase
from Products.CMFPortalLayout.LayoutTool import registerLayoutType
from Products.CMFPortalLayout.utils import getUniqueId, getIdFromPath

class CalendarBox(LayoutBase):
    """
    An object containing the information for creating a calenda box.
    """
    _itemCount = 10
    _subjects = []
    _types = []
    _sort_criteria ='Date'
    _sort_order ='descending'
    _search_in = 'folder'
    _edit_method = 'calendarbox_edit_form'
    _view_method = 'calendarbox_view'
    _standalone_method = 'calendarbox_standalone_view'
    _format = ''
    _useLangInSearch = 0

    security = ClassSecurityInfo()
    meta_type = 'Calendar Box'

    def __init__(self, title=''): #, itemCount=0, subjects=[], types_dict={}, sort_criteria='', sort_order=None, format_types=[], language=''):
        """
        Create a new list box object.
        """
        self.id = getUniqueId()
        self.setTitle(title)

    security.declareProtected(CMFCorePermissions.ManageProperties, 'edit')
    def edit(self, REQUEST):
        """
        Set the menus properties
        """
        zLOG.LOG('ListMenu', zLOG.INFO, 'edit: REQUEST: %s' % (str(REQUEST)))
        self.setTitle(REQUEST.get('title', None))
        self.setItemCount(REQUEST.get('itemCount', None))
        self.setSubjects(REQUEST.get('subjects', None))
        self.setTypes(REQUEST.get('types', None))
        self.setSearch(REQUEST.get('search_in', None))
        self.setSortCriteria(REQUEST.get('sort_criteria', None))
        self.setCatalogSortOrder(REQUEST.get('sort_order', None))
        self.setFormat(REQUEST.get('format', None))
        self.setLanguage(REQUEST.get('item_language', None))
        self.setSearchLanguage(REQUEST.get('use_language', None))

        parent = self.getParent()
        msg = 'Calendar box saved'
        req = REQUEST
        returnurl = req.get('returnToUrl', None)
        if not returnurl:
            returnurl = "%s/edit_layout_columns" % parent.absolute_url()
        req.RESPONSE.redirect('%s?title=%s&render_title=%s&sel_portal_type=%s&item_language=%s&edit_layoutob=%s' % \
                ( returnurl
                , req.get('title', '')
                , req.get('render_title', '')
                , req.get('sel_portal_type','')
                , req.get('item_language','')
                , req.get('edit_layoutob', '')))

    security.declarePublic('getItemCount')
    def getItemCount(self):
        """
        Return the item count for the menu object.
        (How many items will show up in the lists).
        """
        return int(self._itemCount)

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setItemCount')
    def setItemCount(self, itemCount=None):
        """
        Update the item count of the menu object.
        Defines how many items will show up in the day list.
        """
        if itemCount is not None:
            try:
                self._itemCount = int(itemCount)
            except:
                self._itemCount = 0

    security.declarePublic('getFormat')
    def getFormat(self):
        """
        Return the format
        """
        if not hasattr(self, '_format'):
            self._format = ''
        return self._format

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setFormat')
    def setFormat(self, format=None):
        """
        Update the item count of the menu object.
        Defines how many items will show up in the day list.
        """
        if format is not None:
            self._format = format

    security.declarePublic('getSubjects')
    def getSubjects(self):
        """
        Returns the subjects for the calendarbox object.
        """
        return self._subjects and self._subjects or []

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setSubjects')
    def setSubjects(self, subjects=None):
        """

        """
        if subjects is not None:
            if type(subjects) is type(''):
                subjects = subjects.split('\n')
            new_subjects = []
            for subject in subjects:
                s = subject.strip()
                if s:
                    new_subjects.append(s)
            self._subjects = new_subjects

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setSearch')
    def setSearch(self, search_in=None):
        """
        Update where to search.
        """
        if search_in is not None:
            self._search_in = search_in

    security.declarePublic('getSearch')
    def getSearch(self):
        """
        Returns a string saying where to search ('folder','subfolders','site').
        """
        return self._search_in

    security.declarePublic('getTypes')
    def getTypes(self):
        """
        Return the types selected for the menu object.
        Contains a list of the portal types that should turn up in the menu.
        """
        return self._types

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setTypes')
    def setTypes(self, types=None):
        """
        Update the portal types that will turn up in the calendar box.
        """
        if types is not None:
            self._types = types

    security.declarePublic('getSortCriteria')
    def getSortCriteria(self):
        """
        Returns the sort criteria used for this calendarbox object.
        """
        return self._sort_criteria

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setSortCriteria')
    def setSortCriteria(self, sort_criteria=None):
        """
        Update the sort criteria used for this calendarbox object.
        """
        if sort_criteria is not None:
            self._sort_criteria = sort_criteria

    security.declarePublic('getCatalogSortOrder')
    def getCatalogSortOrder(self):
        """
        Returns the sort order used for this object.
        """
        return self._sort_order

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setCatalogSortOrder')
    def setCatalogSortOrder(self, sort_order=None):
        """
        Update the sort order used for this object.
        """
        if sort_order is not None:
            self._sort_order = sort_order

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setSearchLanguage')
    def setSearchLanguage(self, use_language=None):
        """
        Determine whether the language settings should be used in searches
        """
        if use_language is not None:
            self._useLangInSearch = 1
        else:
            self._useLangInSearch = 0

    security.declarePublic('useLanguageInSearch')
    def useLanguageInSearch(self):
        """
        Return a boolean indication whether the language setting should be used in searches
        """
        return self._useLangInSearch

    security.declarePublic('listItems')
    def listItems(self, REQUEST=None, date=None):
        """
        Return the items for the menu
        """
        result = None
        searchpars = {}
        if date is None:
            date = DateTime()
        if type(date) == type(''):
            date = DateTime(date)
        search = self.getSearch()
        if search in ('folder','subfolders'):
            searchpars['path'] = '/'.join(self.getParent().getPhysicalPath())
            #path = self.getParent().getPhysicalPath()
            #if path:
            #    searchpars['path'] = string.join(path, '/')
        if self.getTypes():
            searchpars['Type'] = self.getTypes()
        else:
            searchpars['Type'] = self.portal_types.objectIds()
        if self.getSubjects():
            searchpars['Subject'] = self.getSubjects()
        if self.getCatalogSortOrder():
            searchpars['sort_order'] = self.getCatalogSortOrder()
        searchpars['sort_on'] = self.getSortCriteria()

        if self.useLanguageInSearch():
            if self.getLanguage() is not None:
                searchpars['Language'] = self.getLanguage()

        if self.getTypes():
            if 'Live Event' in self.getTypes():
                searchpars['start_date'] = date.latestTime()
                searchpars['start_date_usage'] = 'range:max'
                searchpars['end_date'] = date.earliestTime()
                searchpars['end_date_usage'] = 'range:min'
            elif 'Event' in self.getTypes():
                searchpars['start'] = date.latestTime()
                searchpars['start_usage'] = 'range:max'
                searchpars['end'] = date.earliestTime()
                searchpars['end_usage'] = 'range:min'
            elif 'Zug Event' in self.getTypes():
                searchpars['start'] = date.latestTime()
                searchpars['start_usage'] = 'range:max'
                searchpars['end'] = date.earliestTime()
                searchpars['end_usage'] = 'range:min'
            else:
                searchpars['modified'] = [date.earliestTime(), date.latestTime()]
                searchpars['modified_usage'] = 'range:min:max'
        else:
            searchpars['modified'] = [date.earliestTime(), date.latestTime()]
            searchpars['modified_usage'] = 'range:min:max'

        try:
            catalog = getToolByName(self, 'portal_catalog')
            result = catalog(searchpars)

            if search == 'folder':
                # this is hackish but we have to remove the items that are not directly in this folder
                res = []
                for brain in result:
                    if "%s/%s" % (searchpars['path'],getIdFromPath(brain.getPath())) == brain.getPath():
                        res.append(brain)
                result = res

            items = []
            return result
            for brain in result:
                items.append(
                            {
                            "path":portalizePath(self, brain.getPath())
                            ,"Title":(brain.Title or getIdFromPath(brain.getPath()))
                            }
                            )
            return items
        except:
            return []

    security.declarePublic('getPrevDate')
    def getPrevDate(self, date=None, mode='month'):
        """
        Returns the previous date used for creating links to previous month/day
        depending on the mode used when calling the function.
        """
        if date is None:
            date = DateTime()
        else:
            if type(date) == type(''):
                date = DateTime(date)

        if mode is None:
            mode = 'month'

        year = date.year()
        month = date.month()
        day = date.day()

        if mode == 'day':
            rangem = month - 1
            monthrange = calendar.monthrange(year, rangem)
            lastday = monthrange[1]
            if month == 1 and day == 1:
                nyear = year - 1
                nmonth = 12
                nday = 31
            elif day == 1:
                nyear = year
                nmonth = month - 1
                nday = lastday
            else:
                nyear = year
                nmonth = month
                nday = day - 1
            ndate = DateTime(nyear, nmonth, nday)
        elif mode == 'month':
            if month == 1:
                nyear = year - 1
                nmonth = 12
            else:
                nyear = year
                nmonth = month - 1
            ndate = DateTime(nyear, nmonth, 1)
        return ndate

    security.declarePublic('getNextDate')
    def getNextDate(self, date=None, mode='month'):
        """
        Returns the next date used for creating links to next month/day
        depending on the mode used when calling the function.
        """
        if date is None:
            date = DateTime()
        else:
            if type(date) == type(''):
                date = DateTime(date)

        if mode is None:
            mode = 'month'

        year = date.year()
        month = date.month()
        day = date.day()

        if mode == 'day':
            monthrange = calendar.monthrange(year, month)
            lastday = monthrange[1]
            if month == 12 and day == 31:
                nyear = year + 1
                nmonth = 1
                nday = 1
            elif day == monthrange[1]:
                nyear = year
                nmonth = month + 1
                nday = 1
            else:
                nyear = year
                nmonth = month
                nday = day + 1
            ndate = DateTime(nyear, nmonth, nday)
        elif mode == 'month':
            if month == 12:
                nyear = year + 1
                nmonth = 1
            else:
                nyear = year
                nmonth = month + 1
            ndate = DateTime(nyear, nmonth, 1)
        return ndate

    security.declarePublic('getHeadline')
    def getHeadline(self, date=None, mode='month'):
        """
        Return the headline for the calendarbox object.
        If you have Localizer installed, this function will translate the day- and monthnames.
        """
        if date is None:
            date = DateTime()
        else:
            if type(date) == type(''):
                date = DateTime(date)

        if mode is None:
            mode = 'month'

        msgc = self.getMessageCatalog()

        if mode == 'day':
            dname = date.Day()
            zLOG.LOG('Calendar Box', zLOG.INFO, 'Dayname: %s' % str(dname))
            if msgc:
                dname = msgc.gettext(dname)
            daydate = date.dd()
            zLOG.LOG('Calendar Box', zLOG.INFO, 'Day date: %s' % str(daydate))
            mname = date.Month()
            zLOG.LOG('Calendar Box', zLOG.INFO, 'Monthname: %s' % str(mname))
            if msgc:
                mname = msgc.gettext(mname)
            yname = date.year()
            zLOG.LOG('Calendar Box', zLOG.INFO, 'Yearname: %s' % str(yname))
            zLOG.LOG('Calendar Box', zLOG.INFO, 'Getting day names')
            zLOG.LOG('Calendar Box', zLOG.INFO, 'Dayname: %s' % str(dname))
            zLOG.LOG('Calendar Box', zLOG.INFO, 'Day date: %s' % str(daydate))
            zLOG.LOG('Calendar Box', zLOG.INFO, 'Monthname: %s' % str(mname))
            zLOG.LOG('Calendar Box', zLOG.INFO, 'Yearname: %s' % str(yname))
            mname = str(dname) + ' ' + str(daydate) + '<br />' + str(mname) + ' ' + str(yname)
        elif mode == 'month':
            mname = date.Month()
            if msgc:
                mname = msgc.gettext(mname)
            year = date.year()
            mname = str(mname) + ' ' + str(year)
        else:
            zLOG.LOG('Box Tool - get headline', zLOG.INFO, 'Mode is: %s' % mode)
        return mname

    security.declarePublic('getDateDict')
    def getDateDict(self, date=None, mode='month'):
        """
        Return a dictionary with the dates for displaying either a month view or a day view.
        The dictionary is build with 2 entries: 'daynames' and 'daylist'.
        The 'daynames' entry contains a list of the weekdays (if month view) or the name of the weekday (if day view).
        The 'daylist' entry contains a list of weeks which contains a list of day dictionaries.
        These day dictionaries contains 2 entries: 'daynr' and 'date'.
        The 'daynr' entry contains the number of the day for displaying in the calendar.
        The 'date' entry contains the actual date for the daynr entry.
        The lists will be created for both month view and date view.
        If you have Localizer installed, the day names will be translated.
        """
        if date is None:
            date = DateTime()
        else:
            if type(date) == type(''):
                date = DateTime(date)

        if mode is None:
            mode = 'month'

        list = {}
        
        msgc = self.getMessageCatalog()

        if mode == 'day':
            dayname = date.Day()
            if msgc:
                dayname = msgc.gettext(dayname)
            daynames = []
            list['daynames'] = daynames.append(dayname)
            montharr = []
            week = []
            day = {}
            daynr = date.day()
            date = date
            day['daynr'] = daynr
            day['date'] = date
            week.append(day)
            montharr.append(week)
            list['daylist'] = montharr
        elif mode == 'month':
            calendar.setfirstweekday(calendar.MONDAY)
            if msgc:
                daynames = []
                daynames.append(msgc.gettext('Mo'))
                daynames.append(msgc.gettext('Tu'))
                daynames.append(msgc.gettext('We'))
                daynames.append(msgc.gettext('Th'))
                daynames.append(msgc.gettext('Fr'))
                daynames.append(msgc.gettext('Sa'))
                daynames.append(msgc.gettext('Su'))
            else:
                daynames = ['Mo', 'Tu', 'We', 'Th', 'Fr', 'Sa', 'Su']
            list['daynames'] = daynames
            year = int(date.year())
            month = int(date.mm())
            monthlist = calendar.monthcalendar(year, month)
            montharr = []
            for weekentry in monthlist:
                week = []
                for dayentry in weekentry:
                    day = {}
                    if dayentry == 0:
                        day['daynr'] = ''
                        day['date'] = None
                        week.append(day)
                    else:
                        day['daynr'] = dayentry
                        day['date'] = DateTime(year, month, dayentry)
                        week.append(day)
                montharr.append(week)
            list['daylist'] = montharr
        return list

registerLayoutType(CalendarBox)
