##############################################################################
#    Copyright (C) 2002  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import os, zLOG, sys
from Globals import InitializeClass, package_home
from AccessControl import getSecurityManager, ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions
from DateTime import DateTime
from Products.CMFPortalLayout.LayoutBase import LayoutBase
from Products.CMFPortalLayout.LayoutTool import registerLayoutType
from Products.CMFPortalLayout.utils import getUniqueId

class ContentMenu(LayoutBase):
    """
    A menu containing poortal content
    """
    _contentpaths = []
    _edit_method = 'contentmenu_edit_form'
    _view_method = 'contentmenu_view'

    security = ClassSecurityInfo()
    type = 'Content Menu'
    meta_type = 'Content Menu'

    def __init__(self, title=''):
        """
        Create a new content menu object.
        """
        self.id = getUniqueId()
        self.setTitle(title)

    security.declareProtected(CMFCorePermissions.ManageProperties, 'edit')
    def edit(self, REQUEST):
        """
        Set the menus properties
        """
        self.setTitle(REQUEST.get('title', None))
        #self.setContent(REQUEST.get('sel_portal_object', None))
        self.setLanguage(REQUEST.get('item_language', None))
        self.setRenderTitle(REQUEST.get('render_title', None))
        self.setRenderHeader(REQUEST.get('render_header', None))

        parent = self.getParent()
        msg = 'Saved'
        req = REQUEST
        returnurl = req.get('returnToUrl', None)
        if not returnurl:
            returnurl = "%s/edit_layout_columns" % parent.absolute_url()
        req.RESPONSE.redirect('%s?title=%s&render_title=%s&render_header=%s&sel_portal_type=%s&item_language=%s&edit_layoutob=%s' % \
                ( returnurl
                , req.get('title', '')
                , req.get('render_title', '')
                , req.get('render_header', '')
                , req.get('sel_portal_type','')
                , req.get('item_language','')
                , req.get('edit_layoutob', '')))

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setContentId')
    def setContent(self, contentpath=None, REQUEST=None):
        """
        Set the id of the object, the box should contain.
        """
        req = REQUEST
        zLOG.LOG('ContentMenu:setContent', zLOG.INFO, 'contentpath: %s' % contentpath)
        if contentpath is not None:
            paths = self._contentpaths
            if contentpath not in paths:
                paths.append(contentpath)
            self._contentpaths = paths
            self._p_changed = 1

        if req is not None:
            returnurl = req.get('returnToUrl', None)
            if not returnurl:
                returnurl = '%s/contentmenu_edit_form' % self.absolute_url()
            req.RESPONSE.redirect('%s?title=%s&render_title=%s&sel_portal_type=%s&item_language=%s&edit_layoutob=%s' % \
                ( returnurl
                , req.get('title', '')
                , req.get('render_title', '')
                , req.get('sel_portal_type','')
                , req.get('item_language','')
                , req.get('edit_layoutob', '')))

    security.declarePublic('getContentPaths')
    def getContentPaths(self):
        """
        Return the content path list
        """
        return self._contentpaths

    security.declareProtected(CMFCorePermissions.ManageProperties, 'deletePath')
    def deletePath(self, index, REQUEST=None):
        """
        Delete the requested object path
        """
        index = int(index)
        paths = self._contentpaths
        del paths[index]
        self._contentpaths = paths
        self._p_changed
        
        if REQUEST is not None:
            req = REQUEST
            parent = self.getParent()
            msg = 'Deleted content'
            returnurl = req.get('returnToUrl', None)
            if not returnurl:
                returnurl = "%s/edit_layout_columns" % parent.absolute_url()
            req.RESPONSE.redirect('%s?edit_layoutob=%s' % (returnurl, req.get('edit_layoutob', '')))
        else:
            self.REQUEST.RESPONSE.redirect('%s/contentmenu_edit_form?portal_status_message=Deleted' % self.absolute_url())

    security.declarePublic('getContentObject')
    def getContentObject(self, path):
        """
        Return the object for the path.
        """
        zLOG.LOG('ContentMenu', zLOG.INFO, 'getContentObject')
        ob = None
        p = getToolByName(self, 'portal_url').getPortalObject()
        try:
            ob = p.restrictedTraverse(path)
        except:
            zLOG.LOG('ContentMenu', zLOG.INFO, 'getContentObject - Exception: %s - %s' % \
                         (sys.exc_info()[0], sys.exc_info()[1]))
            ob = None
            pass

        return ob

    security.declareProtected(CMFCorePermissions.ManageProperties, 'moveContentUp')
    def moveContentUp(self, pidx, REQUEST=None):
        """
        Move the requested path 1 one
        """
        pidx = int(pidx)
        paths = self.getContentPaths()
        if pidx > 0 and pidx < len(paths):
            old = paths[pidx - 1]
            paths[pidx - 1] = paths[pidx]
            paths[pidx] = old
        self._contentpaths = paths
        self._p_changed

        req = REQUEST
        if req is not None:
            returnurl = req.get('returnToUrl', None)
            if not returnurl:
                returnurl = '%s/contentmenu_edit_form' % self.absolute_url()
            req.RESPONSE.redirect('%s?title=%s&render_title=%s&sel_portal_type=%s&item_language=%s&edit_layoutob=%s' % \
                ( returnurl
                , req.get('title', '')
                , req.get('render_title', '')
                , req.get('sel_portal_type','')
                , req.get('item_language','')
                , req.get('edit_layoutob', '')))

    security.declareProtected(CMFCorePermissions.ManageProperties, 'moveContentDown')
    def moveContentDown(self, pidx, REQUEST=None):
        """
        Move the requested path 1 down
        """
        pidx = int(pidx)
        paths = self.getContentPaths()
        if pidx < len(paths)-1 and pidx >= 0:
            old = paths[pidx + 1]
            paths[pidx + 1] = paths[pidx]
            paths[pidx] = old
        self._contentspaths = paths
        self._p_changed = 1

        req = REQUEST
        if req is not None:
            returnurl = req.get('returnToUrl', None)
            if not returnurl:
                returnurl = '%s/contentmenu_edit_form' % self.absolute_url()
            req.RESPONSE.redirect('%s?title=%s&render_title=%s&sel_portal_type=%s&item_language=%s&edit_layoutob=%s' % \
                ( returnurl
                , req.get('title', '')
                , req.get('render_title', '')
                , req.get('sel_portal_type','')
                , req.get('item_language','')
                , req.get('edit_layoutob', '')))


registerLayoutType(ContentMenu)
