##############################################################################
#    Copyright (C) 2002  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import os, zLOG, sys
from Globals import InitializeClass, package_home
from AccessControl import getSecurityManager, ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions
from DateTime import DateTime
from Products.CMFPortalLayout.LayoutBase import LayoutBase
from Products.CMFPortalLayout.LayoutTool import registerLayoutType
from Products.CMFPortalLayout.utils import getUniqueId

class ImageMenu(LayoutBase):
    """
    A menu containing image content
    """
    _contentpath = None
    _edit_method = 'imagemenu_edit_form'
    _view_method = 'imagemenu_view'
    _link = None

    security = ClassSecurityInfo()
    type = 'Image Menu'
    meta_type = 'Image Menu'

    def __init__(self, title=''):
        """
        Create a new image menu object.
        """
        self.id = getUniqueId()
        self.setTitle(title)

    security.declareProtected(CMFCorePermissions.ManageProperties, 'edit')
    def edit(self, REQUEST):
        """
        Set the menus properties
        """
        self.setTitle(REQUEST.get('title', None))
        self.setContent(REQUEST.get('content_path', None))
        self.setLink(REQUEST.get('link', None))
        self.setRenderTitle(REQUEST.get('render_title', None))
        self.setRenderHeader(REQUEST.get('render_header', None))

        parent = self.getParent()
        msg = 'Saved'
        req = REQUEST
        returnurl = req.get('returnToUrl', None)
        if not returnurl:
            returnurl = "%s/edit_layout_columns" % parent.absolute_url()
        req.RESPONSE.redirect('%s?title=%s&content_path=%s&link=%s&render_title=%s&render_header=%s&edit_layoutob=%s' % \
                ( returnurl
                , req.get('title', '')
                , req.get('content_path', '')
                , req.get('link', '')
                , req.get('render_title', '')
                , req.get('render_header', '')
                , req.get('edit_layoutob', '')))

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setContent')
    def setContent(self, contentpath=None):
        """
        Set the id of the object, the box should contain.
        """
        if contentpath is not None:
            self._contentpath = contentpath

    security.declarePublic('getContentPath')
    def getContentPath(self):
        """
        Return the content path list
        """
        return self._contentpath

    security.declarePublic('getContentObject')
    def getContentObject(self):
        """
        Return the object for the path.
        """
        ob = None
        p = getToolByName(self, 'portal_url').getPortalObject()
        if self.getContentPath():
            try:
                ob = p.restrictedTraverse(self.getContentPath())
            except:
                zLOG.LOG('ContentMenu', zLOG.INFO, 'getContentObject - Exception: %s - %s' % \
                         (sys.exc_info()[0], sys.exc_info()[1]))
                ob = None
                pass

        return ob
        
    security.declarePrivate('setLink')
    def setLink(self, link=None):
        """
        Set the link
        """
        if link is not None:
            self._link = link
            
    security.declarePublic('getLink')
    def getLink(self):
        """
        Return the link
        """
        return self._link

registerLayoutType(ImageMenu)
