##############################################################################
#    Copyright (C) 2003  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import os, zLOG, sys
from Globals import InitializeClass, package_home
from AccessControl import getSecurityManager, ClassSecurityInfo
from OFS.Folder import Folder
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions
from DateTime import DateTime
from Products.CMFPortalLayout.LayoutBase import LayoutBase
from Products.CMFPortalLayout.LayoutTool import registerLayoutType
from Products.CMFPortalLayout.utils import getUniqueId

class LayoutBox(Folder, LayoutBase):
    """
    A menu containing layout columns and objects
    """
    _columns = []
    _edit_method = 'layoutbox_edit_form'
    _view_method = 'layoutbox_view'

    security = ClassSecurityInfo()
    type = 'Layout Box'
    meta_type = 'Layout Box'

    def __init__(self, title=''):
        """
        Create a new layout box object.
        """
        self.id = getUniqueId()
        self.setTitle(title)

    security.declareProtected(CMFCorePermissions.ManageProperties, 'edit')
    def edit(self, REQUEST):
        """
        Set the menus properties
        """
        self.setTitle(REQUEST.get('boxtitle', None))

        parent = self.getParent()
        msg = 'Saved'
        req = REQUEST
        returnurl = req.get('returnToUrl', None)
        if not returnurl:
            returnurl = "%s/edit_layout_columns" % parent.absolute_url()

        req.RESPONSE.redirect('%s?boxtitle=%s&edit_layoutob=%s' % \
                    (returnurl
                    , req.get('boxtitle', '')
                    , req.get('edit_layoutob', '')))

    security.declareProtected(CMFCorePermissions.ManageProperties, 'addColumn')
    def addColumn(self, REQUEST=None):
        """
        Add a new column to the layout object
        """
        cid = getUniqueId()
        self.manage_addProduct['OFSP'].manage_addFolder(cid,title="Layout Column")
        column = getattr(self, cid)
        columns = self._columns
        portal_url = getToolByName(self, 'portal_url')
        newpath = portal_url.getRelativeContentURL(column)
        columns.append(newpath)
        self._columns = columns
        self._p_changed = 1

        if REQUEST is not None:
            req = REQUEST
            parent = self.getParent()
            msg = 'Saved'
            returnurl = req.get('returnToUrl', None)
            if not returnurl:
                returnurl = "%s/edit_layout_columns" % parent.absolute_url()
            req.RESPONSE.redirect('%s?title=%s&render_title=%s&render_header=%s&edit_layoutob=%s' % \
                    ( returnurl
                    , req.get('title', '')
                    , req.get('render_title', '')
                    , req.get('render_header', '')
                    , req.get('edit_layoutob', '')))

    security.declareProtected(CMFCorePermissions.ManageProperties, 'editColumn')
    def editColumn(self, REQUEST):
        """
        Edit the requested column
        """
        req = REQUEST
        column = None
        columnid = req.get('columnid', None)
        if columnid:
            column = getattr(self, columnid, None)
        if not column:
            raise AttributeError, 'Could not find requested column'

        title = req.get('columntitle', None)
        width = req.get('column_width', None)
        if width == '0':
            width = None
        if not column.hasProperty('width'):
            column.manage_addProperty(id='width', value=None, type='string')
        column.manage_changeProperties(title=title, width=width)

        parent = self.getParent()
        msg = 'Saved'
        returnurl = req.get('returnToUrl', None)
        if not returnurl:
            returnurl = "%s/edit_layout_columns" % parent.absolute_url()
        req.RESPONSE.redirect('%s?columntitle=%s&render_title=%s&render_header=%s&edit_layoutob=%s' % \
                ( returnurl
                , req.get('columntitle', '')
                , req.get('render_title', '')
                , req.get('render_header', '')
                , req.get('edit_layoutob', '')))

    security.declarePublic('listColumns')
    def listColumns(self):
        """
        Return the columns
        """
        columns = []
        portal = getToolByName(self, 'portal_url').getPortalObject()
        cpaths = self._columns
        for cpath in cpaths:
            column = portal.restrictedTraverse(cpath)
            columns.append(column)
        return columns

    security.declareProtected(CMFCorePermissions.ManageProperties, 'deleteColumn')
    def deleteColumn(self, columnid, REQUEST=None):
        """
        Delete the requested column
        """
        portal_url = getToolByName(self, 'portal_url')
        try:
            column = getattr(self, columnid)
        except:
            column = None
        if column:
            columns = self._columns
            columns.remove(portal_url.getRelativeContentURL(column))
            self._delObject(column.getId())
            self._columns = columns
            self._p_changed = 1

        if REQUEST is not None:
            req = REQUEST
            parent = self.getParent()
            msg = 'Saved'
            returnurl = req.get('returnToUrl', None)
            if not returnurl:
                returnurl = "%s/edit_layout_columns" % parent.absolute_url()
            req.RESPONSE.redirect('%s?title=%s&render_title=%s&render_header=%s&edit_layoutob=%s' % \
                    ( returnurl
                    , req.get('title', '')
                    , req.get('render_title', '')
                    , req.get('render_header', '')
                    , req.get('edit_layoutob', '')))

    security.declareProtected(CMFCorePermissions.ManageProperties, 'moveColumn')
    def moveColumn(self, cidx, direction, REQUEST=None):
        """
        Move the requested column
        """
        cidx = int(cidx)
        columns = self._columns
        if direction == 'up':
            if cidx > 0 and cidx < len(columns):
                old = columns[cidx - 1]
                columns[cidx - 1] = columns[cidx]
                columns[cidx] = old
        elif direction == 'down':
            if cidx < len(columns)-1 and cidx >= 0:
                old = columns[cidx + 1]
                columns[cidx + 1] = columns[cidx]
                columns[cidx] = old
        else:
            raise AttributeError, 'Unknown direction: %s' % direction

        self._columns = columns
        self._p_changed = 1

        if REQUEST is not None:
            req = REQUEST
            parent = self.getParent()
            msg = 'Saved'
            returnurl = req.get('returnToUrl', None)
            if not returnurl:
                returnurl = "%s/edit_layout_columns" % parent.absolute_url()
            req.RESPONSE.redirect('%s?title=%s&render_title=%s&render_header=%s&edit_layoutob=%s' % \
                    ( returnurl
                    , req.get('title', '')
                    , req.get('render_title', '')
                    , req.get('render_header', '')
                    , req.get('edit_layoutob', '')))

    security.declareProtected(CMFCorePermissions.ManageProperties, 'addLayoutToColumn')
    def addLayoutToColumn(self, columnid, layouttype, title=None, REQUEST=None):
        """
        Add a new layout object in the requested column
        """
        column = getattr(self, columnid, None)
        if not column:
            raise AttributeError, 'Requested column with id %s could not be found'
        ltool = getToolByName(self, 'portal_layout')
        layout = None
        for lt in ltool.listLayoutClasses():
            if lt.meta_type == layouttype:
                layout = lt(title)
                break

        if layout is None:
            # No menu type matched passed in value
            raise NameError, 'Unknown layout Type: %s' % layouttype
        try:
            column._setObject(layout.getId(), layout)
        except:
            zLOG.LOG('LayoutBox', zLOG.INFO, 'Exception in addLayoutToColumn: Type: %s - Value: %s' % \
                   (sys.exc_info()[0], sys.exc_info()[1]))

        if REQUEST is not None:
            req = REQUEST
            parent = self.getParent()
            msg = 'Saved'
            returnurl = req.get('returnToUrl', None)
            if not returnurl:
                returnurl = "%s/edit_layout_columns" % parent.absolute_url()
            req.RESPONSE.redirect('%s?title=%s&render_title=%s&render_header=%s&edit_layoutob=%s' % \
                    ( returnurl
                    , req.get('title', '')
                    , req.get('render_title', '')
                    , req.get('render_header', '')
                    , req.get('edit_layoutob', '')))

    security.declarePublic('listLayoutsForColumn')
    def listLayoutsForColumn(self, column):
        """
        List sorted layout objects for requested column
        """
        layouts = []
        ltool = getToolByName(self, 'portal_layout')
        if column:
            if column.objectValues():
                layouts = list(column.objectValues(ltool.listLayoutTypes()))
                layouts.sort()
        return layouts

    security.declarePublic('listLayoutsInColumnForLanguage')
    def listLayoutsInColumnForLanguage(self, column):
        """
        Returns all layout items created in column
        Uses language to determine what items dhould be returned
        """
        layouts = self.listLayoutsForColumn(column)
        ltool = getToolByName(self, 'portal_layout')
        try:
            msgcatalog = self.restrictedTraverse(ltool.getMessageCatalog())
            sel_language = msgcatalog.get_selected_language()
        except:
            sel_language = 'en'
        for layout in layouts:
            lan = layout.getLanguage()
            if lan and lan != sel_language:
                layouts.remove(layout)
        return layouts

    security.declareProtected(CMFCorePermissions.ManageProperties, 'deleteLayoutInColumn')
    def deleteLayoutInColumn(self, columnid, layoutids, REQUEST=None):
        """ Delete selected layout(s) """
        column = getattr(self, columnid, None)
        if not column:
            raise AttributeError, 'Requested column with id %s could not be found'

        if type(layoutids) is type(''):
            column._delObject(layoutids)
        elif type(layoutids) in (type(()), type([])):
            for lid in layoutids:
                column._delObject(lid)

        if REQUEST is not None:
            req = REQUEST
            parent = self.getParent()
            msg = 'Saved'
            returnurl = req.get('returnToUrl', None)
            if not returnurl:
                returnurl = "%s/edit_layout_columns" % parent.absolute_url()
            req.RESPONSE.redirect('%s?title=%s&render_title=%s&render_header=%s&edit_layoutob=%s' % \
                    ( returnurl
                    , req.get('title', '')
                    , req.get('render_title', '')
                    , req.get('render_header', '')
                    , req.get('edit_layoutob', '')))

    security.declareProtected(CMFCorePermissions.ManageProperties, 'moveLayoutInColumn')
    def moveLayoutInColumn(self, columnid, layout_id, direction, REQUEST=None):
        """ Move selected layout(s) """
        column = getattr(self, columnid, None)
        if not column:
            raise AttributeError, 'Requested column with id %s could not be found'

        swapobj = None

        swap1 = getattr(column, layout_id)
        #zLOG.LOG('MenuTool', zLOG.INFO, 'moveMenuIn - menu %s' % (swap1.title_or_id()))
        objs = self.listLayoutsForColumn(column)

        if direction == 'up':
            for layout in objs:
                if swapobj is None: swapobj = layout
                if layout.getId() != id:
                    if layout.sort_order < swap1.sort_order and layout.sort_order > swapobj.sort_order:
                        swapobj = layout
            if swapobj is not None:
                tmp = swapobj.sort_order
                swapobj.setSortOrder(swap1.sort_order)
                swap1.setSortOrder(tmp)
        elif direction == 'down':
            for layout in objs:
                if swapobj is None: swapobj = layout
                if layout.getId() != id:
                    if layout.sort_order > swap1.sort_order and layout.sort_order > swapobj.sort_order:
                        swapobj = layout
                        break
            if swapobj is not None:
                tmp = swapobj.sort_order
                swapobj.setSortOrder(swap1.sort_order)
                swap1.setSortOrder(tmp)
        
        if REQUEST is not None:
            req = REQUEST
            parent = self.getParent()
            msg = 'Saved'
            returnurl = req.get('returnToUrl', None)
            if not returnurl:
                returnurl = "%s/edit_layout_columns" % parent.absolute_url()
            req.RESPONSE.redirect('%s?title=%s&render_title=%s&render_header=%s&edit_layoutob=%s' % \
                    ( returnurl
                    , req.get('title', '')
                    , req.get('render_title', '')
                    , req.get('render_header', '')
                    , req.get('edit_layoutob', '')))

registerLayoutType(LayoutBox)
