##############################################################################
#    Copyright (C) 2002  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import os, zLOG
from Globals import InitializeClass, package_home
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import _getViewFor
from DateTime import DateTime
from Products.CMFPortalLayout.LayoutBase import LayoutBase
from Products.CMFPortalLayout.LayoutTool import registerLayoutType
from Products.CMFPortalLayout.utils import getUniqueId, getIdFromPath

class LinkMenu(LayoutBase):
    """
    An object containing the information for creating a link box.
    """
    _edit_method = 'linkmenu_edit_form'
    _view_method = 'linkmenu_view'
    _style = None
    _links = []

    security = ClassSecurityInfo()
    meta_type = 'Link Menu'

    def __init__(self, title=''):
        """
        Create a new list box object.
        """
        self.id = getUniqueId()
        self.setTitle(title)

    security.declareProtected(CMFCorePermissions.ManageProperties, 'edit')
    def edit(self, REQUEST):
        """
        Set the menus properties
        """
        self.setTitle(REQUEST.get('title', None))
        self.setStyle(REQUEST.get('style', None))
        self.setShowupFor(REQUEST.get('showupForType', None))
        self.setLanguage(REQUEST.get('item_language', None))
        self.editLinks(REQUEST, redirect=0)

        parent = self.getParent()
        msg = 'Saved'
        req = REQUEST
        returnurl = req.get('returnToUrl', None)
        if not returnurl:
            returnurl = "%s/edit_layout_columns" % parent.absolute_url()
        req.RESPONSE.redirect('%s?title=%s&render_title=%s&sel_portal_type=%s&item_language=%s&edit_layoutob=%s' % \
                ( returnurl
                , req.get('title', '')
                , req.get('render_title', '')
                , req.get('sel_portal_type','')
                , req.get('item_language','')
                , req.get('edit_layoutob', '')))

    security.declarePublic('getStyle')
    def getStyle(self):
        """
        Return the name style in which the menu entries should be rendered.
        """
        return self._style

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setStyle')
    def setStyle(self, style=None):
        """
        Set the name of the style in which the menu entries should be rendered.
        """
        if style is not None:
            self._style = style

    security.declareProtected(CMFCorePermissions.ManageProperties, 'editLinks')
    def editLinks(self, REQUEST, redirect=1):
        """
        Edit the links
        """
        req = REQUEST
        links = self.getLinks()
        if links:
            index = 0
            for linkdict in links:
                changed = 0
                linktitle = req.get('linktitle_%d' % index, None)
                linktype = req.get('linktype_%d' % index, None)
                linktarget = req.get('linktarget_%d_%s' % (index, linktype), None)
                if linktitle is not None:
                    linkdict['title'] = linktitle
                    changed = 1
                if linktype is not None:
                    linkdict['type'] = linktype
                    changed = 1
                if linktarget is not None:
                    linkdict['target'] = linktarget
                    changed = 1
                if changed:
                    links[index] = linkdict
                index = index + 1
            self._links = links
            self._p_changed = 1
            msg = 'Links saved'
        else:
            msg = 'No links added'

        if redirect:
            parent = self.getParent()

            returnurl = req.get('returnToUrl', None)
            if not returnurl:
                returnurl = "%s/edit_layout_columns" % parent.absolute_url()
            req.RESPONSE.redirect('%s?title=%s&render_title=%s&sel_portal_type=%s&item_language=%s&edit_layoutob=%s' % \
                    ( returnurl
                    , req.get('title', '')
                    , req.get('render_title', '')
                    , req.get('sel_portal_type','')
                    , req.get('item_language','')
                    , req.get('edit_layoutob', '')))

    security.declareProtected(CMFCorePermissions.ManageProperties, 'addLink')
    def addLink(self, REQUEST):
        """
        Add a new link
        """
        zLOG.LOG('Link Menu', zLOG.INFO, 'Request is %s' % str(REQUEST.form))
        req = REQUEST
        links = self.getLinks()
        newtitle = req.get('newlink_title', None)
        newtype = req.get('newlink_type', None)
        newtarget = req.get('newlink_target_%s' % newtype, None)
        if newtitle is not None and newtype is not None and newtarget is not None:
            newdict = {'title': newtitle, 'type': newtype, 'target': newtarget}
            links.append(newdict)
            self._links = links
            self._p_changed = 1
            msg = 'Added new link'
        else:
            msg = 'No data entered for new link'

        parent = self.getParent()

        returnurl = req.get('returnToUrl', None)
        if not returnurl:
            returnurl = "%s/edit_layout_columns" % parent.absolute_url()
        req.RESPONSE.redirect('%s?title=%s&render_title=%s&sel_portal_type=%s&item_language=%s&edit_layoutob=%s' % \
                    ( returnurl
                    , req.get('title', '')
                    , req.get('render_title', '')
                    , req.get('sel_portal_type','')
                    , req.get('item_language','')
                    , req.get('edit_layoutob', '')))

    security.declareProtected(CMFCorePermissions.ManageProperties, 'deleteLink')
    def deleteLink(self, REQUEST):
        """
        Delete a link
        """
        req = REQUEST
        links = self.getLinks()
        del_id = req.get('deleteIndex', None)
        if del_id:
            del_id = int(del_id)
            del links[del_id]
            self._links = links
            self._p_changed = 1
            msg = 'Deleted link'
        else:
            msg = 'No index supplied'

        parent = self.getParent()

        returnurl = req.get('returnToUrl', None)
        if not returnurl:
            returnurl = "%s/edit_layout_columns" % parent.absolute_url()
        req.RESPONSE.redirect('%s?title=%s&render_title=%s&sel_portal_type=%s&item_language=%s&edit_layoutob=%s' % \
                    ( returnurl
                    , req.get('title', '')
                    , req.get('render_title', '')
                    , req.get('sel_portal_type','')
                    , req.get('item_language','')
                    , req.get('edit_layoutob', '')))

    security.declareProtected(CMFCorePermissions.ManageProperties, 'moveupLink')
    def moveLink(self, lidx, direction, REQUEST=None):
        """
        Move the requested link item in direction
        """
        lidx = int(lidx)
        links = self.getLinks()
        if direction == 'up':
            if lidx > 0 and lidx < len(links):
                old = links[lidx - 1]
                links[lidx - 1] = links[lidx]
                links[lidx] = old
        elif direction == 'down':
            if lidx < len(links) and lidx >= 0:
                old = links[lidx + 1]
                links[lidx + 1] = links[lidx]
                links[lidx] = old
        else:
            raise AttributeError, 'Unknown direction: %s' % direction
        self._links = links
        self._p_changed = 1
        
        if REQUEST is not None:
            req = REQUEST
            parent = self.getParent()
            msg = 'Link moved %s' % direction
            returnurl = req.get('returnToUrl', None)
            if not returnurl:
                returnurl = '%s/edit_layout_columns' % parent.absolute_url()
            req.RESPONSE.redirect('%s?title=%s&render_title=%s&edit_layoutob=%s' % \
                (returnurl
               , req.get('title', '')
               , req.get('render_title', '')
               , req.get('edit_layoutob', '')))

    security.declarePublic('getFormat')
    def getFormat(self):
        """
        Return the format
        """
        if not hasattr(self, '_format'):
            self._format = ''
        return self._format

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setFormat')
    def setFormat(self, format=None):
        """
        Update the item count of the menu object.
        Defines how many items will show up in the day list.
        """
        if format is not None:
            self._format = format

    security.declarePublic('getUrlFor')
    def getUrlFor(self, idx):
        """
        Return the url for the link
        """
        links = self.getLinks()
        idx = int(idx)
        linkdict = links[idx]
        linktype = linkdict['type']
        linktarget = linkdict['target']
        if linktype == 'link':
            url = linktarget
        elif linktype in ['content', 'layout']:
            portal = getToolByName(self, 'portal_url').getPortalObject()
            try:
                obj = portal.restrictedTraverse(linktarget)
                url = obj.absolute_url()
            except:
                url = portal.absolute_url()
            if linktype == 'layout':
                url = "%s/renderAlone" % url
        else:
            raise AttributeError, 'Link type %s not valid' % linktype
        return url

    security.declarePublic('getLinks')
    def getLinks(self):
        """
        Return the links
        """
        return self._links

registerLayoutType(LinkMenu)
