##############################################################################
#    Copyright (C) 2002  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import os, zLOG
from Globals import InitializeClass, package_home
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import _getViewFor
from DateTime import DateTime
from Products.CMFPortalLayout.LayoutBase import LayoutBase
from Products.CMFPortalLayout.LayoutTool import registerLayoutType
from Products.CMFPortalLayout.utils import getUniqueId, getIdFromPath

class OrderedMenu(LayoutBase):

    """
    An object containing the information for creating an ordered menu.
    """

    _edit_method = 'orderedmenu_edit_form'
    _view_method = 'orderedmenu_view'
    _format = ''
    _ordered_object_list = []

    security = ClassSecurityInfo()
    meta_type = 'Ordered Menu'

    def __init__(self, title=''):
        """
        Create a new ordered menu object.
        """
        self.id = getUniqueId()
        self.setTitle(title)

    security.declareProtected(CMFCorePermissions.ManageProperties, 'edit')
    def edit(self, REQUEST):
        """
        Set the menus properties
        """
        self.setTitle(REQUEST.get('title', None))
        self.setFormat(REQUEST.get('format', None))
        self.setSelectedObjects(REQUEST.get('sel_objects', None))
        self.setRenderTitle(REQUEST.get('render_title', None))

        parent = self.getParent()
        msg = 'Ordered Menu Saved'
        req = REQUEST
        returnurl = req.get('returnToUrl', None)
        if not returnurl:
            returnurl = "%s/edit_layout_columns" % parent.absolute_url()
        req.RESPONSE.redirect('%s?title=%s&render_title=%s&sel_portal_type=%s&item_language=%s&edit_layoutob=%s' % \
                ( returnurl
                , req.get('title', '')
                , req.get('render_title', '')
                , req.get('sel_portal_type','')
                , req.get('item_language','')
                , req.get('edit_layoutob', '')))

    security.declarePublic('getFormat')
    def getFormat(self):
        """
        Return the format
        """
        if not hasattr(self, '_format'):
            self._format = ''
        return self._format

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setFormat')
    def setFormat(self, format=None):
        """
        Update the item count of the menu object.
        Defines how many items will show up in the day list.
        """
        if format is not None:
            self._format = format

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setSelectedObjects')
    def setSelectedObjects(self, sel_objects=None):
        """
        Set the objects that are selected to show up in the ordered menu
        """
        if sel_objects is not None:
            existing_object_list = self._ordered_object_list
            for dict in existing_object_list:
                if dict['id'] in sel_objects:
                    dict['isListed'] = 1
                else:
                    dict['isListed'] = 0

            self._ordered_object_list = existing_object_list
            self._p_changed = 1

    security.declareProtected( CMFCorePermissions.ManagePortal, 'moveUpObjects' )
    def moveUpObjects( self, indexes=(), REQUEST=None ):
        """
        Move the specified objects up one slot in our list.
        """
        elements = list( map( int, indexes ) )  # Convert to a list of integers.
        elements.sort()

        ordered_list = self._ordered_object_list

        for idx in elements:
            idx2 = idx - 1
            if idx2 < 0:
                # Wrap to the bottom.
                idx2 = len(ordered_list) - 1
            # Swap.
            a = ordered_list[idx2]
            ordered_list[idx2] = ordered_list[idx]
            ordered_list[idx] = a

        self._ordered_object_list = ordered_list
        self._p_changed = 1

        req = REQUEST
        if req is not None:
            returnurl = req.get('returnToUrl', None)
            if not returnurl:
                returnurl = '%s/contentmenu_edit_form' % self.absolute_url()
            req.RESPONSE.redirect('%s?title=%s&render_title=%s&sel_portal_type=%s&item_language=%s&edit_layoutob=%s' % \
                ( returnurl
                , req.get('title', '')
                , req.get('render_title', '')
                , req.get('sel_portal_type','')
                , req.get('item_language','')
                , req.get('edit_layoutob', '')))

    security.declareProtected( CMFCorePermissions.ManagePortal, 'moveDownObjects' )
    def moveDownObjects( self, indexes=(), REQUEST=None ):
        """
        Move the specified objects down one slot in our list.
        """
        elements = list( map( int, indexes ) )  # Convert to a list of integers.
        elements.sort()
        elements.reverse()

        ordered_list = self._ordered_object_list

        for idx in elements:
            idx2 = idx + 1
            if idx2 >= len(ordered_list):
                # Wrap to the top.
                idx2 = 0
            # Swap.
            a = ordered_list[idx2]
            ordered_list[idx2] = ordered_list[idx]
            ordered_list[idx] = a

        self._ordered_object_list = ordered_list

        req = REQUEST
        if req is not None:
            returnurl = req.get('returnToUrl', None)
            if not returnurl:
                returnurl = '%s/contentmenu_edit_form' % self.absolute_url()
            req.RESPONSE.redirect('%s?title=%s&render_title=%s&sel_portal_type=%s&item_language=%s&edit_layoutob=%s' % \
                ( returnurl
                , req.get('title', '')
                , req.get('render_title', '')
                , req.get('sel_portal_type','')
                , req.get('item_language','')
                , req.get('edit_layoutob', '')))

    security.declareProtected(CMFCorePermissions.ManageProperties, 'getObjectList')
    def getObjectList(self):
        """
        Return a list of dictionaries containing object informations for objects in folder.
        """
        menutool = getToolByName(self, 'portal_layout')
        folder = self.getParent()
        objects = folder.contentValues()
        a_types = menutool.listAvailableTypes()
        existing_list = self._ordered_object_list
        new_list = []

        objectIdsInFolder = []
        for object in objects:
            if object.Type() in a_types:
                objectIdsInFolder.append(object.id)

        for dict in existing_list:
            dictid = dict['id']
            if dictid in objectIdsInFolder:
                new_list.append(dict)

        for objectid in objectIdsInFolder:
            new_id = 1
            for dict in existing_list:
                dictid = dict['id']
                if dictid == objectid:
                    new_id = 0
            if new_id:
                object = getattr(folder, objectid, None)
                if object:
                    new_dict = {'id': object.id, 'title': object.title, 'isListed': 0}
                    new_list.append(new_dict)

        self._ordered_object_list = new_list
        self._p_changed = 1
        return self._ordered_object_list

    security.declarePublic('listItems')
    def listSelectedObjects(self):
        """
        Return the selected objects for the menu
        """
        folder = self.getParent()
        objectlist = []
        for dict in self._ordered_object_list:
            if dict['isListed']:
                obj = getattr(folder, dict['id'], None)
                if obj:
                    objectlist.append(obj)

        return objectlist


registerLayoutType(OrderedMenu)
