##############################################################################
#    Copyright (C) 2002  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import os, sys, traceback, string, threading
import time
import urllib
import xmllib
from zLOG import LOG, INFO, ERROR

from urllib import quote_plus
from OFS.Traversable import Traversable

from Globals import InitializeClass, PersistentMapping, package_home
from AccessControl import ClassSecurityInfo
from Acquisition import ImplicitAcquisitionWrapper, Implicit, aq_base, aq_inner, aq_parent
from Persistence import Persistent
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import _getViewFor
from Products.CMFCore.interfaces.DublinCore import DublinCore
from DateTime import DateTime
from Products.CMFPortalLayout.LayoutBase import LayoutBase
from Products.CMFPortalLayout.LayoutTool import registerLayoutType
from Products.CMFPortalLayout.utils import getUniqueId, getIdFromPath

# Portions of this Code namely known_elements and RDFSummary is from RDFSummary version 1.0.
# Small changes by Thomas Olsen <tol@tanghus.dk>.
#
# The Initial Developer of the RDFSummary version 1.0 Code is European Environment
# Agency (EEA).  Portions created by CMG are
# Copyright (C) European Environment Agency.  All
# Rights Reserved.
#
# Contributor(s):
# Soren Roug, EEA
# Dominique Dutoit, European Commission
# Tomas Hjelmberg, CMG
# URAGO Keisuke, <bravo@resourcez.org> - for slash code


known_elements = {
# Old Netscape RSS elements version 0.9
  'http://my.netscape.com/rdf/simple/0.9/ title':'title',
  'http://my.netscape.com/rdf/simple/0.9/ description': 'description',
  'http://my.netscape.com/rdf/simple/0.9/ link': 'link',
  'http://my.netscape.com/rdf/simple/0.9/ name': 'name',
  'http://my.netscape.com/rdf/simple/0.9/ url': 'url',
# Netscape RSS elements version 0.91 (No namespace)
  'title':'title',
  'description': 'description',
  'link': 'link',
  'url': 'url',
  'name': 'name',
  'rating':'rating',
  'language':'language',
  'width': 'width',
  'height': 'height',
  'copyright':'copyright',
  'pubDate':'pubdate',
  'lastBuildDate':'lastbuilddate',
  'docs':'docs',
  'managingEditor':'managingeditor',
  'webMaster':'webmaster',
  'hour':'hour',
  'day':'day',
  'skipDays':'skipdays',
  'skipHours':'skiphours',
# Netscape RSS elements version 0.92 (No namespace)
  'cloud':'cloud',
  'source':'source',
  'enclosure':'enclosure',
  'category':'category',
# Base RSS 1.0 elements
  'http://purl.org/rss/1.0/ title':'title',
  'http://purl.org/rss/1.0/ description': 'description',
  'http://purl.org/rss/1.0/ link': 'link',
  'http://purl.org/rss/1.0/ name': 'name',
  'http://purl.org/rss/1.0/ url': 'url',
  'http://purl.org/rss/1.0/ li': 'li', # added by tol. This doesn't work but prevents errors on som feeds.
# RSS Syndication
  'http://purl.org/rss/1.0/modules/syndication/ updatePeriod': 'updatePeriod',
  'http://purl.org/rss/1.0/modules/syndication/ updateFrequency': 'updateFrequency',
  'http://purl.org/rss/1.0/modules/syndication/ updateBase': 'updateBase',
# Dublin Core
  'http://purl.org/dc/elements/1.1/ title':'title',
  'http://purl.org/dc/elements/1.1/ creator':'creator',
  'http://purl.org/dc/elements/1.1/ subject':'subject',
  'http://purl.org/dc/elements/1.1/ description':'description',
  'http://purl.org/dc/elements/1.1/ publisher':'publisher',
  'http://purl.org/dc/elements/1.1/ contributor':'contributor',
  'http://purl.org/dc/elements/1.1/ date':'date',
  'http://purl.org/dc/elements/1.1/ type':'type',
  'http://purl.org/dc/elements/1.1/ format':'format',
  'http://purl.org/dc/elements/1.1/ identifier':'identifier',
  'http://purl.org/dc/elements/1.1/ source':'source',
  'http://purl.org/dc/elements/1.1/ language':'language',
  'http://purl.org/dc/elements/1.1/ relation':'relation',
  'http://purl.org/dc/elements/1.1/ coverage':'coverage',
  'http://purl.org/dc/elements/1.1/ rights':'rights',
# Obsolete Dublin Core
  'http://purl.org/metadata/dublin_core# Title':'title',
  'http://purl.org/metadata/dublin_core# Creator':'creator',
  'http://purl.org/metadata/dublin_core# Subject':'subject',
  'http://purl.org/metadata/dublin_core# Description':'description',
  'http://purl.org/metadata/dublin_core# Publisher':'publisher',
  'http://purl.org/metadata/dublin_core# Contributor':'contributor',
  'http://purl.org/metadata/dublin_core# Date':'date',
  'http://purl.org/metadata/dublin_core# Type':'type',
  'http://purl.org/metadata/dublin_core# Format':'format',
  'http://purl.org/metadata/dublin_core# Identifier':'identifier',
  'http://purl.org/metadata/dublin_core# Source':'source',
  'http://purl.org/metadata/dublin_core# Language':'language',
  'http://purl.org/metadata/dublin_core# Relation':'relation',
  'http://purl.org/metadata/dublin_core# Coverage':'coverage',
  'http://purl.org/metadata/dublin_core# Rights':'rights',
# My own Event module
  'http://purl.org/rss/1.0/modules/event/ startdate': 'startdate',
  'http://purl.org/rss/1.0/modules/event/ enddate': 'enddate',
  'http://purl.org/rss/1.0/modules/event/ location': 'location',
  'http://purl.org/rss/1.0/modules/event/ type': 'type',
  'http://purl.org/rss/1.0/modules/event/ organizer': 'organizer',
# Slash module (as abused by Newsforge)
  'http://slashcode.com/rss/1.0/modules/Slash/ slash': 'slash',
  'http://slashcode.com/rss/1.0/modules/Slash/ department': 'department',
  'http://slashcode.com/rss/1.0/modules/Slash/ section': 'section',
  'http://slashcode.com/rss/1.0/modules/Slash/ comments': 'comments',
  'http://slashcode.com/rss/1.0/modules/Slash/ hitparade': 'hitparade',
# Slash module (as defined in standard documentation)
  'http://slashcode.com/rss/1.0/modules/slash/ department': 'department',
  'http://slashcode.com/rss/1.0/modules/slash/ section': 'section',
  'http://slashcode.com/rss/1.0/modules/slash/ comments': 'comments',
  'http://slashcode.com/rss/1.0/modules/slash/ hit_parade': 'hitparade',
# Meta Vocab - for www.zopelabs.com
  'http://webns.net/mvcb/ errorReportsTo': 'errorReportsTo', # added by tol
  }

class RDFParser(xmllib.XMLParser):
    """Parse an RSS file"""

    def __init__(self):
        self.inside_channel = 0
        self.channel = {}
        self.textinput = {}
        self.items = []
        self.image = {}
        self.encoding = 'UTF-8'
        self._rssver = 100
        self.__item={}
        self.__data=[]
        self.__count=0
    #   self.entitydefs = known_entities
        self.elements = {
        # RSS 1.0
          'http://purl.org/rss/1.0/ item':
              (self.start_item,self.end_item),
          'http://purl.org/rss/1.0/ items':
              (self.ignore_tag,self.ignore_tag),
          'http://purl.org/rss/1.0/ image':
              (self.start_image,self.end_image),
          'http://purl.org/rss/1.0/ channel':
              (self.start_channel,self.end_channel),
          'http://purl.org/rss/1.0/ textinput':
              (self.start_textinput,self.end_textinput),
        # Netscape RSS 0.9
          'http://my.netscape.com/rdf/simple/0.9/ item':
              (self.start_item,self.end_item),
          'http://my.netscape.com/rdf/simple/0.9/ image':
              (self.start_image,self.end_image),
          'http://my.netscape.com/rdf/simple/0.9/ channel':
              (self.start_channel,self.end_channel),
          'http://my.netscape.com/rdf/simple/0.9/ textinput':
              (self.start_textinput,self.end_textinput),
        # RDF
          'http://www.w3.org/1999/02/22-rdf-syntax-ns# RDF':
              (self.ignore_tag,self.ignore_tag),
          'http://www.w3.org/1999/02/22-rdf-syntax-ns# Seq':
              (self.ignore_tag,self.ignore_tag),
          'http://www.w3.org/1999/02/22-rdf-syntax-ns# Bag':
              (self.ignore_tag,self.ignore_tag),
          'http://www.w3.org/1999/02/22-rdf-syntax-ns# Alt':
              (self.ignore_tag,self.ignore_tag),
          'http://www.w3.org/1999/02/22-rdf-syntax-ns# li':
              (self.ignore_tag,self.ignore_tag),
        # RSS 0.91
          'rss': (self.start_rss91,self.ignore_tag),
          'channel': (self.start_channel,self.end_channel),
          'item': (self.start_item,self.end_item),
          'image': (self.start_image,self.end_image),
          'textinput': (self.start_textinput,self.end_textinput),
          # For the benefit of my.userland.com
          'textInput': (self.start_textinput,self.end_textinput),
          }
        xmllib.XMLParser.__init__(self)

# Most tags don't have special handlers. What we do is to recognize
# <item>, <channel> and <image> and <textinput>
# tags inside those gets converted to properties of the handler tag.

    def handle_xml(self,encoding,standalone):
        if(encoding):
            self.encoding = encoding

    def unknown_starttag(self, tag, attrs):
        if known_elements.has_key(tag):
            self.__data = []
            self.__tag=known_elements[tag]

    def unknown_endtag(self, tag):
        if not known_elements.has_key(tag):
            raise RuntimeError, "Unsupported tag: " + tag
        self.__item[self.__tag]=string.join(self.__data, "")

    def ignore_tag(self,attrs=None):
        pass

    def handle_data(self, text):
        self.__data.append(text)

    def handle_cdata(self, text):
        self.__data.append(text)

    def handle_charref(self,ref):
        self.handle_data('&#' + ref + ';')

    def unknown_entityref(self,ref):
        self.handle_data('&' + ref + ';')

    def syntax_error(self,message):
        pass

    def start_channel(self, attrs):
        "Start of channel info"
        self.inside_channel = 1
        self.channel = {}
        self.items = []
        self.image = {}
        self.__item= {}

    def end_channel(self):
        self.channel=self.__item
        self.channel['encoding'] = self.encoding
        self.inside_channel = 0

    def start_textinput(self, attrs):
        if self._rssver == 91:
            self.__itemsave = self.__item
        self.__item={}

    def end_textinput(self):
        self.textinput=self.__item
        if self._rssver == 91:
            self.__item = self.__itemsave

    def start_item(self, attrs):
        if self._rssver == 91:
            self.__itemsave = self.__item
        self.__item={}

    def end_item(self):
        self.items.append(self.__item)
        if self._rssver == 91:
            self.__item = self.__itemsave

    def start_image(self, attrs):
        if self._rssver != 91 and self.inside_channel:
            return
        if self._rssver == 91:
            self.__itemsave = self.__item
        self.__item={}

    def end_image(self):
        """
        this basically involves fetching the image file into a data string.
        """
        if self._rssver != 91 and self.inside_channel:
            return


        u = urllib.URLopener()
        if not u:
            raise IOError, "Unsupported protocol"
        u.addheader("User-agent", "RDFSummary (helpdesk@eionet.eu.int)")
        f = u.open(self.__item['url'])

        if not f:
            raise IOError, "Failure in open"
        data = f.read()
        headers = f.info()
        f.close()
        if headers.has_key('content-type'):
            ctype=headers['content-type']
        else:
            ctype='image/gif'

        # now to import the image data
        self.image=self.__item
        self.image['data'] = data
        self.image['content_type'] = ctype
        self.image['size'] = len(data)
        # restore the itemstore to how it was before <image> if RSS 0.91
        if self._rssver == 91:
            self.__item = self.__itemsave

    def parse_url(self, url):
        """
        Grab the file from the webserver and feed it to the parser
        """
        u = urllib.URLopener()
        if not u:
            raise IOError, "Unsupported protocol"
        u.addheader("User-agent", "RDFSummary (helpdesk@eionet.eu.int)")
        f = u.open(url)

        if not f:
            raise IOError, "Failure in open"
        self.rdfsource = f.read()
        del u # I had dangling <CLOSE-WAIT> connections before this

        if self.rdfsource:
            self.feed(self.rdfsource)
        else:
            raise RuntimeError, "Unable to GET content"

# RSS 0.91 support
    def start_rss91(self, attrs):
        self._rssver = 91

class RSSItem(Persistent):
    """
        semi-persistent class which provides Dublin Core methods.
        TODO: constructor should receive both the channel dict and 
        the item dict directly and decide itself what to do with it.
    """
    __implements__ = DublinCore
    
    meta_type = 'RSS Item'

    security = ClassSecurityInfo()
    id = None
    title = ''
    creator = None
    subject = ()
    description = ''
    publisher = None
    contributors = ()
    date = None
    effective_date = None
    expiration_date = None
    format = 'text/html'
    language = None
    rights = None
    identifier = None
    sort_order = 0
    
    def __init__( self
                , id
                , sort_order
                , title=None
                , subject=None
                , description=None
                , contributors=None
                , effective_date=None
                , expiration_date=None
                , identifier=None
                , format='text/html'
                , language=''
                , rights=None
                ):
                
        self.id = id
        self.sort_order = sort_order
                
        if title is not None:
            self.title = title

        if subject is not None:
            self.subject = subject
    
        if description is not None:
            self.description = description
    
        if contributors is not None:
            self.contributors = contributors
    
        if effective_date is not None:
            self.effective_date = effective_date

        if expiration_date is not None:
            self.expiration_date = expiration_date

        if identifier is not None:
            self.identifier = identifier

        if format is not None:
            self.format = format

        if language is not None:
            self.language = language

        if rights is not None:
            self.rights = rights

    def __repr__(self):
        return "<RSSItem %s: %s>" % (self.id, self.title)
    
    def __of__(self, parent):
        '''
        '''
        w_self = ImplicitAcquisitionWrapper(aq_base(self), parent)
        return w_self

    def __cmp__(self, other):
        """ Compare function - used when sorting elements """
        if other is not None:
            return cmp(self.sort_order, other.sort_order)
        else:
            return 1

    security.declarePublic('getId')
    def getId(self):
        return self.id
    
    
    security.declarePublic('Title')
    def Title(self):
        """
            Dublin Core element - resource name

            Return type: string
            Permissions: View
        """
        return self.title
        
    security.declarePublic('title_or_id')
    def title_or_id(self):
        """
            Dublin Core element - resource name

            Return type: string
            Permissions: View
        """
        return self.title and self.title or self.id
        
    security.declarePublic('Creator')
    def Creator(self):
        """
            Dublin Core element - resource creator

            Return the full name(s) of the author(s) of the content object.

            Return type: string
            Permission: View
        """
        return self.creator

    security.declarePublic('Subject')
    def Subject(self):
        """
            Dublin Core element - resource keywords

            Return zero or more keywords associated with the content object.

            Return type: sequence of strings
            Permission: View
        """
        return self.subject

    security.declarePublic('Description')
    def Description(self):
        """
            Dublin Core element - resource summary

            Return a natural language description of this object.

            Return type: string
            Permissions: View
        """
        return self.description

    security.declarePublic('Publisher')
    def Publisher(self):
        """
            Dublin Core element - resource publisher

            Return full formal name of the entity or person responsible
            for publishing the resource.

            Return type: string
            Permission: View
        """
        return self.publisher

    security.declarePublic('Contributors')
    def Contributors(self):
        """
            Dublin Core element - resource collaborators

            Return zero or additional collaborators.

            Return type: sequence of strings
            Permission: View
        """
        return self.contributors
    
    security.declarePublic('Date')
    def Date(self):
        """
            Dublin Core element - default date

            Return type: string, formatted 'YYYY-MM-DD H24:MN:SS TZ'
            Permissions: View
        """
        return self.date
    
    security.declarePublic('CreationDate')
    def CreationDate(self):
        """
            Dublin Core element - date resource created.

            Return type: string, formatted 'YYYY-MM-DD H24:MN:SS TZ'
            Permissions: View
        """
        return self.Date()
    
    security.declarePublic('EffectiveDate')
    def EffectiveDate(self):
        """
            Dublin Core element - date resource becomes effective.

            Return type: string, formatted 'YYYY-MM-DD H24:MN:SS TZ'
            Permissions: View
        """
        return self.Date()
    
    security.declarePublic('ExpirationDate')
    def ExpirationDate(self):
        """
            Dublin Core element - date resource expires.

            Return type: string, formatted 'YYYY-MM-DD H24:MN:SS TZ'
            Permissions: View
        """
        return self.Date()
    
    security.declarePublic('ModificationDate')
    def ModificationDate(self):
        """
            Dublin Core element - date resource last modified.

            Return type: string, formatted 'YYYY-MM-DD H24:MN:SS TZ'
            Permissions: View
        """
        return self.Date()

    security.declarePublic('Type')
    def Type(self):
        """
            Dublin Core element - resource type

            Return a human-readable type name for the resource
            (perhaps mapped from its Zope meta_type).

            Return type: string
            Permissions: View
        """
        return 'RSS Item'

    security.declarePublic('Format')
    def Format(self):
        """
            Dublin Core element - resource format

            Return the resource's MIME type (e.g., 'text/html',
            'image/png', etc.).

            Return type: string
            Permissions: View
        """
        return self.format

    security.declarePublic('Identifier')
    def Identifier(self):
        """
            Dublin Core element - resource ID

            Returns unique ID (a URL) for the resource.

            Return type: string
            Permissions: View
        """
        return self.identifier

    security.declarePublic('Language')
    def Language(self):
        """
            Dublin Core element - resource language

            Return the RFC language code (e.g., 'en-US', 'pt-BR')
            for the resource.

            Return type: string
            Permissions: View
        """
        return self.language
        
    security.declarePublic('absolute_url')
    def absolute_url(self):
        return self.Identifier()



class RSSMenu(Traversable, Implicit, LayoutBase):
    """
    An object containing the information for creating a list of
    predefined searches.
    """
    _updating = 0
    _itemCount = 10
    _url = None
    _link = None
    _items = None
    _edit_method = 'rssmenu_edit_form'
    _view_method = 'rssmenu_view'
    _format = ''
    _lastupdate = None
    
    isPrincipiaFolderish = 1

    security = ClassSecurityInfo()
    meta_type = 'RSS Menu'

    def __init__(self, title=''): 
        """
        Create a new RSS menu object.
        """
        self.id = getUniqueId()
        self.setTitle(title)
        self._items = PersistentMapping()

    security.declareProtected( CMFCorePermissions.View, 'getItem' )
    def getItem( self, item_id ):
        """
            Return an rss item, given its ID;  raise KeyError
            if not found.
        """
        #LOG(self.meta_type, INFO, 'getItem: %s - %s' % (item_id, self._items.get( item_id )))
        return self._items.get( item_id ).__of__(self)

    security.declarePublic('Description')
    def Description(self):
        """
        Return the description of the feed.
        """
        return self.description

    security.declarePublic('Link')
    def Link(self):
        """
        Return the link to the site.
        """
        return self._link

    # Is this right?
    security.declareProtected( CMFCorePermissions.View, '__bobo_traverse__' )
    def __bobo_traverse__(self, REQUEST, name):
        """
        This will make this container traversable
        """
        #LOG(self.meta_type, INFO, 'bobo_traverse: %s' % name)
        if name in self.objectIds():
            obj = self.getItem(name)
            #LOG(self.meta_type, INFO, 'bobo_traverse: %s, returning from items' % obj)
            return obj
        #LOG(self.meta_type, INFO, 'bobo_traverse: %s, returning default' % name)
        parent = aq_parent( aq_inner( self ) )
        if hasattr(parent, '__bobo_traverse__'):    
            return parent.__bobo_traverse__(REQUEST, name)
        else:
            return getattr(self, name)

    #
    #   OFS.ObjectManager query interface.
    #
    security.declareProtected( CMFCorePermissions.AccessContentsInformation
                             , 'objectIds' )
    def objectIds( self, spec=None ):
        """
            Return a list of the ids of our DiscussionItems.
        """
        #LOG(self.meta_type, INFO, 'objectIds')
        if self._updating:
            return []
        if spec and spec is not RSSItem.meta_type:
            return []
        return self._items.keys()


    security.declareProtected( CMFCorePermissions.AccessContentsInformation
                             , 'objectItems' )
    def objectItems(self, spec=None):
        """
            Return a list of (id, subobject) tuples for our DiscussionItems.
        """
        #LOG(self.meta_type, INFO, 'objectItems')
        if self._updating:
            return []
        r=[]
        a=r.append
        g=self.getItem
        for id in self.objectIds(spec):
            a( (id, g( id ) ) )
        return r


    security.declareProtected( CMFCorePermissions.AccessContentsInformation
                             , 'objectValues' )
    def objectValues(self):
        """
            Return a list of our DiscussionItems.
        """
        if self._updating:
            return []
        vals = []
        for key, val in self.objectItems():
            vals.append(val)
        #LOG(self.meta_type, INFO, 'objectValues')
        return vals

    security.declareProtected(CMFCorePermissions.ManageProperties, 'edit')
    def edit(self, REQUEST):
        """
        Set the menus properties
        """
        #LOG('RSSMenu', INFO, 'edit: REQUEST: %s' % (str(REQUEST)))
        self.setTitle(REQUEST.get('title', None))
        self.setItemCount(REQUEST.get('itemCount', None))
        self.setURL(REQUEST.get('url', None))
        self.setFormat(REQUEST.get('format', None))
        self.setRenderTitle(REQUEST.get('render_title', None))

        self._lastupdate = None # trigger update
        
        parent = self.getParent()
        msg = 'Saved'
        req = REQUEST
        returnurl = req.get('returnToUrl', None)
        if not returnurl:
            returnurl = "%s/edit_layout_columns" % parent.absolute_url()
        req.RESPONSE.redirect('%s?title=%s&render_title=%s&sel_portal_type=%s&item_language=%s&edit_layoutob=%s' % \
                ( returnurl
                , req.get('title', '')
                , req.get('render_title', '')
                , req.get('sel_portal_type','')
                , req.get('item_language','')
                , req.get('edit_layoutob', '')))

    security.declarePublic('getFormat')
    def getFormat(self):
        """
        Return the format
        """
        if not hasattr(self, '_format'):
            self._format = ''
        return self._format

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setFormat')
    def setFormat(self, format=None):
        """
        Update the item count of the menu object.
        Defines how many items will show up in the day list.
        """
        if format is not None:
            self._format = format

    security.declarePublic('getItemCount')
    def getItemCount(self):
        """
        Return the item count for the menu object.
        (How many items will show up in the lists).
        """
        try:
            return int(self._itemCount)
        except:
            LOG(self.meta_type, INFO, 'getItemCount: exception %s - %s\n%s' % (sys.exc_type, sys.exc_value, traceback.format_list(traceback.extract_tb(sys.exc_traceback))))
            return 0

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setItemCount')
    def setItemCount(self, itemCount=None):
        """
        Update the item count of the menu object.
        Defines how many items will show up in the day list.
        """
        if itemCount is not None:
            try:
                self._itemCount = int(itemCount)
            except:
                self._itemCount = 0

    security.declarePublic('getURL')
    def getURL(self):
        """
        Return the url
        """
        if not hasattr(self, '_url'):
            self._url = None
        return self._url

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setURL')
    def setURL(self, url=None):
        """
        Set the url to be used.
        """
        if url is not None:
            self._url = url

    security.declareProtected(CMFCorePermissions.ManageProperties, 'update')
    def update(self):
        """
        Update from the feed.
        TODO: Keep a log of feeds that repeatedly gives parse errors
        and disable them after X tries. There should alse be a GUI
        part for reabling them.
        """
        if not self._url:
            return
        interval = 60*30
        now = int(time.time())
        if self._lastupdate is None or (now - self._lastupdate) > interval:
            #LOG(self.meta_type, INFO, 'update: url: %s' % self._url)
            p=RDFParser()
            try:
                p.parse_url(self._url)
            except:
                LOG(self.meta_type, INFO, 'update: exception parsing %s: %s - %s' % \
                    ( self._url
                    , sys.exc_type
                    , sys.exc_value))
                traceback.print_list(traceback.extract_tb(sys.exc_traceback))
                return
                
            # channel and at least one item is required in all supported RSS-versions
            if p.items != [] and p.channel != {}:
                try:
                    self._updating = 1
                    self._lastupdate = now
                    LOG(self.meta_type, INFO, 'update - channel: %s' % str(p.channel))
                    for key in self._items.keys():
                        del self._items[key]
                    items = PersistentMapping()
                    if p.channel.has_key('title'):
                        self.title = p.channel['title']
                    if p.channel.has_key('description'):
                        self.description = p.channel['description']
                    if p.channel.has_key('link'):
                        self._link = p.channel['link']

                    count = self.getItemCount() or len(p.items)

                    for item in p.items:
                        rssitem = RSSItem(
                                    id=str(len(items))
                                    ,sort_order=len(items)
                                    ,title=item.get('title', None)
                                    ,description=item.get('description', None)
                                    ,identifier=item.get('link', None)
                                    )
                        items[str(len(items))] = rssitem
                        count = count - 1
                        if not count: break
                        #LOG(self.meta_type, INFO, 'update - item: %s' % str(item.keys()))
                        #LOG(self.meta_type, INFO, 'update - item: %s' % repr(rssitem))
                    self._items = items
                    self._p_changed = 1
                    self._updating = 0
                except:
                    LOG(self.meta_type, INFO, 'update: exception parsing %s: %s - %s' % \
                        ( self._url
                        , sys.exc_type
                        , sys.exc_value))
                    traceback.print_list(traceback.extract_tb(sys.exc_traceback))
                    self._updating = 0
            else:
                LOG(self.meta_type, ERROR, 'Nothing from: %s' % self._url)
            del p

    
    security.declarePublic('listItems')
    def listItems(self):
        """
        Return the items for the menu
        """
        #LOG(self.meta_type, INFO, 'listItems')
        #self.update()
        items = self.objectValues()
        
        t = threading.Thread( None, self.update, None, () )
        t.start()
            
        items.sort()
        #if self.getItemCount():
        #    items = items[:self.getItemCount()]

        #LOG(self.meta_type, INFO, 'listItems - returning %d items\n %s' % (self.getItemCount(), str(items)))

        return items

registerLayoutType(RSSMenu)
