##############################################################################
#    Copyright (C) 2002  MMmanager.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##############################################################################

import os, zLOG, sys, traceback
from urllib import quote_plus
from Globals import InitializeClass, package_home
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import _getViewFor
from DateTime import DateTime
from Products.CMFPortalLayout.LayoutBase import LayoutBase
from Products.CMFPortalLayout.LayoutTool import registerLayoutType
from Products.CMFPortalLayout.utils import getUniqueId, getIdFromPath

class SubjectSearchMenu(LayoutBase):
    """
    An object containing the information for creating a list of
    predefined searches.
    """
    _itemCount = 10
    _subjects = []
    _types = []
    _search_in = 'folder'
    _edit_method = 'subjectsearchmenu_edit_form'
    _view_method = 'subjectsearchmenu_view'
    _style = ''

    security = ClassSecurityInfo()
    meta_type = 'Subject Search Menu'

    def __init__(self, title=''): #, itemCount=0, subjects=[], types_dict={}, sort_criteria='', sort_order=None, format_types=[], language=''):
        """
        Create a new subject search object.
        """
        self.id = getUniqueId()
        self.setTitle(title)

    security.declareProtected(CMFCorePermissions.ManageProperties, 'edit')
    def edit(self, REQUEST):
        """
        Set the menus properties
        """
        zLOG.LOG('ListMenu', zLOG.INFO, 'edit: REQUEST: %s' % (str(REQUEST)))
        self.setTitle(REQUEST.get('title', None))
        self.setItemCount(REQUEST.get('itemCount', None))
        self.setTypes(REQUEST.get('types', None))
        self.setSearch(REQUEST.get('search_in', None))
        self.setStyle(REQUEST.get('style', None))
        self.setRenderTitle(REQUEST.get('render_title', None))

        parent = self.getParent()
        msg = 'Saved'
        req = REQUEST
        returnurl = req.get('returnToUrl', None)
        if not returnurl:
            returnurl = "%s/edit_layout_columns" % parent.absolute_url()
        req.RESPONSE.redirect('%s?title=%s&render_title=%s&sel_portal_type=%s&item_language=%s&edit_layoutob=%s' % \
                ( returnurl
                , req.get('title', '')
                , req.get('render_title', '')
                , req.get('sel_portal_type','')
                , req.get('item_language','')
                , req.get('edit_layoutob', '')))

    security.declarePublic('getItemCount')
    def getItemCount(self):
        """
        Return the item count for the menu object.
        (How many items will show up in the lists).
        """
        try:
            return int(self._itemCount)
        except:
            zLOG.LOG(self.meta_type, zLOG.INFO, 'getItemCount: exception %s - %s\n%s' % (sys.exc_type, sys.exc_value, traceback.format_list(traceback.extract_tb(sys.exc_traceback))))
            return 0

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setItemCount')
    def setItemCount(self, itemCount=None):
        """
        Update the item count of the menu object.
        Defines how many items will show up in the day list.
        """
        if itemCount is not None:
            try:
                self._itemCount = int(itemCount)
            except:
                self._itemCount = 0

    security.declarePublic('getStyle')
    def getStyle(self):
        """
        Return the style
        """
        if not hasattr(self, '_style'):
            self._style = ''
        return self._style

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setStyle')
    def setStyle(self, style=None):
        """
        Set the style to be used.
        """
        if style is not None:
            self._style = style

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setSearch')
    def setSearch(self, search_in=None):
        """
        Update where to search.
        """
        if search_in is not None:
            self._search_in = search_in

    security.declarePublic('getSearch')
    def getSearch(self):
        """
        Returns a string saying where to search ('folder','subfolders','site').
        """
        return self._search_in

    security.declarePublic('getTypes')
    def getTypes(self):
        """
        Return the types selected for the menu object.
        Contains a list of the portal types that should turn up in the menu.
        """
        return self._types

    security.declareProtected(CMFCorePermissions.ManageProperties, 'setTypes')
    def setTypes(self, types=None):
        """
        Update the portal types that will turn up in the calendar box.
        """
        if types is not None:
            self._types = types

    security.declarePublic('listItems')
    def listItems(self):
        """
        Return the items for the menu
        """
        items = []
        parent = self.getParent()
        if self.getTypes():
            subjects = self.getSubjectsForTypes(self.getTypes())
        else:
            return items
        type_str = ''
        for typ in self.getTypes():
            type_str = type_str + "&portal_type:list=%s" % quote_plus(typ)
        search = self.getSearch()
        if search in ('folder','subfolders'):
            zLOG.LOG(self.meta_type, zLOG.INFO, 'path %s' % (str(self.aq_inner.aq_parent.getPhysicalPath())))
            path = '/'.join(parent.getPhysicalPath())
        else:
            path = '/'

        for subject in subjects:
            qsubject = quote_plus(subject)
            items.append(
                        {
                         "url":"%s/doSearch?path=%s&Subject=%s%s" % (parent.absolute_url(), path, qsubject, type_str)
                        ,"title":subject
                        }
                       )
        if self.getItemCount():
            items = items[:self.getItemCount()]

        zLOG.LOG(self.meta_type, zLOG.INFO, 'listItems - returning %d items\n %s' % (self.getItemCount(), str(items)))

        return items

registerLayoutType(SubjectSearchMenu)
